@@ -27,6 +27,7 @@
 
 #include <winpr/crt.h>
 #include <winpr/assert.h>
+#include <winpr/cast.h>
 #include <winpr/print.h>
 #include <winpr/stream.h>
 
@@ -68,7 +69,8 @@ static UINT rdpsnd_server_send_formats(RdpsndServerContext* context)
 	Stream_Write_UINT32(s, 0);                           /* dwVolume */
 	Stream_Write_UINT32(s, 0);                           /* dwPitch */
 	Stream_Write_UINT16(s, 0);                           /* wDGramPort */
-	Stream_Write_UINT16(s, context->num_server_formats); /* wNumberOfFormats */
+	Stream_Write_UINT16(
+	    s, WINPR_ASSERTING_INT_CAST(uint16_t, context->num_server_formats)); /* wNumberOfFormats */
 	Stream_Write_UINT8(s, context->block_no);            /* cLastBlockConfirmed */
 	Stream_Write_UINT16(s, CHANNEL_VERSION_WIN_MAX);     /* wVersion */
 	Stream_Write_UINT8(s, 0);                            /* bPad */
@@ -82,13 +84,18 @@ static UINT rdpsnd_server_send_formats(RdpsndServerContext* context)
 	}
 
 	const size_t pos = Stream_GetPosition(s);
+	if (pos > UINT16_MAX)
+		goto fail;
+
+	WINPR_ASSERT(pos >= 4);
 	Stream_SetPosition(s, 2);
-	Stream_Write_UINT16(s, pos - 4);
+	Stream_Write_UINT16(s, (UINT16)(pos - 4));
 	Stream_SetPosition(s, pos);
 
 	WINPR_ASSERT(context->priv);
-	status = WTSVirtualChannelWrite(context->priv->ChannelHandle, (PCHAR)Stream_Buffer(s),
-	                                Stream_GetPosition(s), &written);
+
+	status = WTSVirtualChannelWrite(context->priv->ChannelHandle, Stream_BufferAs(s, char),
+	                                (UINT32)pos, &written);
 	Stream_SetPosition(s, 0);
 fail:
 	return status ? CHANNEL_RC_OK : ERROR_INTERNAL_ERROR;
@@ -391,6 +398,8 @@ static UINT rdpsnd_server_select_format(RdpsndServerContext* context, UINT16 cli
 			if (context->priv->out_frames < bs)
 				context->priv->out_frames = bs;
 
+			break;
+		default:
 			break;
 	}
 
@@ -427,7 +436,6 @@ out:
 static UINT rdpsnd_server_training(RdpsndServerContext* context, UINT16 timestamp, UINT16 packsize,
                                    BYTE* data)
 {
-	size_t end = 0;
 	ULONG written = 0;
 	BOOL status = 0;
 	wStream* s = rdpsnd_server_get_buffer(context);
@@ -452,12 +460,15 @@ static UINT rdpsnd_server_training(RdpsndServerContext* context, UINT16 timestam
 		Stream_Write(s, data, packsize);
 	}
 
-	end = Stream_GetPosition(s);
-	Stream_SetPosition(s, 2);
-	Stream_Write_UINT16(s, end - 4);
+	const size_t end = Stream_GetPosition(s);
+	if ((end < 4) || (end > UINT16_MAX))
+		return ERROR_INTERNAL_ERROR;
 
-	status = WTSVirtualChannelWrite(context->priv->ChannelHandle, (PCHAR)Stream_Buffer(s), end,
-	                                &written);
+	Stream_SetPosition(s, 2);
+	Stream_Write_UINT16(s, (UINT16)(end - 4));
+
+	status = WTSVirtualChannelWrite(context->priv->ChannelHandle, Stream_BufferAs(s, char),
+	                                (UINT32)end, &written);
 
 	Stream_SetPosition(s, 0);
 
@@ -492,10 +503,6 @@ static BOOL rdpsnd_server_align_wave_pdu(wStream* s, UINT32 alignment)
  */
 static UINT rdpsnd_server_send_wave_pdu(RdpsndServerContext* context, UINT16 wTimestamp)
 {
-	size_t length = 0;
-	size_t start = 0;
-	size_t end = 0;
-	const BYTE* src = NULL;
 	AUDIO_FORMAT* format = NULL;
 	ULONG written = 0;
 	UINT error = CHANNEL_RC_OK;
@@ -520,29 +527,31 @@ static UINT rdpsnd_server_send_wave_pdu(RdpsndServerContext* context, UINT16 wTi
 	Stream_Write_UINT16(s, context->selected_client_format); /* wFormatNo */
 	Stream_Write_UINT8(s, context->block_no);                /* cBlockNo */
 	Stream_Seek(s, 3);                                       /* bPad */
-	start = Stream_GetPosition(s);
-	src = context->priv->out_buffer;
-	length = 1ull * context->priv->out_pending_frames * context->priv->src_bytes_per_frame;
+	const size_t start = Stream_GetPosition(s);
+	const BYTE* src = context->priv->out_buffer;
+	const size_t length =
+	    1ull * context->priv->out_pending_frames * context->priv->src_bytes_per_frame;
 
 	if (!freerdp_dsp_encode(context->priv->dsp_context, context->src_format, src, length, s))
 		return ERROR_INTERNAL_ERROR;
-	else
+
+	/* Set stream size */
+	if (!rdpsnd_server_align_wave_pdu(s, format->nBlockAlign))
+		return ERROR_INTERNAL_ERROR;
+
+	const size_t end = Stream_GetPosition(s);
+	const size_t pos = end - start + 8ULL;
+	if (pos > UINT16_MAX)
+		return ERROR_INTERNAL_ERROR;
+	Stream_SetPosition(s, 2);
+	Stream_Write_UINT16(s, (UINT16)pos);
+	Stream_SetPosition(s, end);
+
+	if (!WTSVirtualChannelWrite(context->priv->ChannelHandle, Stream_BufferAs(s, char),
+	                            (UINT32)(start + 4), &written))
 	{
-		/* Set stream size */
-		if (!rdpsnd_server_align_wave_pdu(s, format->nBlockAlign))
-			return ERROR_INTERNAL_ERROR;
-
-		end = Stream_GetPosition(s);
-		Stream_SetPosition(s, 2);
-		Stream_Write_UINT16(s, end - start + 8);
-		Stream_SetPosition(s, end);
-
-		if (!WTSVirtualChannelWrite(context->priv->ChannelHandle, (PCHAR)Stream_Buffer(s),
-		                            start + 4, &written))
-		{
-			WLog_ERR(TAG, "WTSVirtualChannelWrite failed!");
-			error = ERROR_INTERNAL_ERROR;
-		}
+		WLog_ERR(TAG, "WTSVirtualChannelWrite failed!");
+		error = ERROR_INTERNAL_ERROR;
 	}
 
 	if (error != CHANNEL_RC_OK)
@@ -556,8 +565,9 @@ static UINT rdpsnd_server_send_wave_pdu(RdpsndServerContext* context, UINT16 wTi
 	Stream_Write_UINT32(s, 0); /* bPad */
 	Stream_SetPosition(s, start);
 
-	if (!WTSVirtualChannelWrite(context->priv->ChannelHandle, Stream_Pointer(s), end - start,
-	                            &written))
+	WINPR_ASSERT((end - start) <= UINT32_MAX);
+	if (!WTSVirtualChannelWrite(context->priv->ChannelHandle, Stream_Pointer(s),
+	                            (UINT32)(end - start), &written))
 	{
 		WLog_ERR(TAG, "WTSVirtualChannelWrite failed!");
 		error = ERROR_INTERNAL_ERROR;
@@ -581,7 +591,6 @@ static UINT rdpsnd_server_send_wave2_pdu(RdpsndServerContext* context, UINT16 fo
                                          const BYTE* data, size_t size, BOOL encoded,
                                          UINT16 timestamp, UINT32 audioTimeStamp)
 {
-	size_t end = 0;
 	ULONG written = 0;
 	UINT error = CHANNEL_RC_OK;
 	BOOL status = 0;
@@ -633,16 +642,22 @@ static UINT rdpsnd_server_send_wave2_pdu(RdpsndServerContext* context, UINT16 fo
 		}
 	}
 
-	end = Stream_GetPosition(s);
-	Stream_SetPosition(s, 2);
-	Stream_Write_UINT16(s, end - 4);
+	const size_t end = Stream_GetPosition(s);
+	if (end > UINT16_MAX + 4)
+	{
+		error = ERROR_INTERNAL_ERROR;
+		goto out;
+	}
 
-	status = WTSVirtualChannelWrite(context->priv->ChannelHandle, (PCHAR)Stream_Buffer(s), end,
-	                                &written);
+	Stream_SetPosition(s, 2);
+	Stream_Write_UINT16(s, (UINT16)(end - 4));
+
+	status = WTSVirtualChannelWrite(context->priv->ChannelHandle, Stream_BufferAs(s, char),
+	                                (UINT32)end, &written);
 
 	if (!status || (end != written))
 	{
-		WLog_ERR(TAG, "WTSVirtualChannelWrite failed! [stream length=%" PRIdz " - written=%" PRIu32,
+		WLog_ERR(TAG, "WTSVirtualChannelWrite failed! [stream length=%" PRIuz " - written=%" PRIu32,
 		         end, written);
 		error = ERROR_INTERNAL_ERROR;
 	}
@@ -761,7 +776,6 @@ static UINT rdpsnd_server_send_samples2(RdpsndServerContext* context, UINT16 for
  */
 static UINT rdpsnd_server_set_volume(RdpsndServerContext* context, UINT16 left, UINT16 right)
 {
-	size_t len = 0;
 	BOOL status = 0;
 	ULONG written = 0;
 	wStream* s = rdpsnd_server_get_buffer(context);
@@ -774,9 +788,10 @@ static UINT rdpsnd_server_set_volume(RdpsndServerContext* context, UINT16 left,
 	Stream_Write_UINT16(s, 4); /* Payload length */
 	Stream_Write_UINT16(s, left);
 	Stream_Write_UINT16(s, right);
-	len = Stream_GetPosition(s);
 
-	status = WTSVirtualChannelWrite(context->priv->ChannelHandle, (PCHAR)Stream_Buffer(s),
+	const size_t len = Stream_GetPosition(s);
+	WINPR_ASSERT(len <= UINT32_MAX);
+	status = WTSVirtualChannelWrite(context->priv->ChannelHandle, Stream_BufferAs(s, char),
 	                                (ULONG)len, &written);
 	Stream_SetPosition(s, 0);
 	return status ? CHANNEL_RC_OK : ERROR_INTERNAL_ERROR;
@@ -789,7 +804,6 @@ static UINT rdpsnd_server_set_volume(RdpsndServerContext* context, UINT16 left,
  */
 static UINT rdpsnd_server_close(RdpsndServerContext* context)
 {
-	size_t pos = 0;
 	BOOL status = 0;
 	ULONG written = 0;
 	UINT error = CHANNEL_RC_OK;
@@ -823,12 +837,16 @@ static UINT rdpsnd_server_close(RdpsndServerContext* context)
 	Stream_Write_UINT8(s, SNDC_CLOSE);
 	Stream_Write_UINT8(s, 0);
 	Stream_Seek_UINT16(s);
-	pos = Stream_GetPosition(s);
+	const size_t pos = Stream_GetPosition(s);
+	WINPR_ASSERT(pos >= 4);
 	Stream_SetPosition(s, 2);
-	Stream_Write_UINT16(s, pos - 4);
+	Stream_Write_UINT16(s, WINPR_ASSERTING_INT_CAST(uint16_t, pos - 4));
 	Stream_SetPosition(s, pos);
-	status = WTSVirtualChannelWrite(context->priv->ChannelHandle, (PCHAR)Stream_Buffer(s),
-	                                Stream_GetPosition(s), &written);
+
+	const size_t len = Stream_GetPosition(s);
+	WINPR_ASSERT(len <= UINT32_MAX);
+	status = WTSVirtualChannelWrite(context->priv->ChannelHandle, Stream_BufferAs(s, char),
+	                                (UINT32)len, &written);
 	Stream_SetPosition(s, 0);
 	return status ? CHANNEL_RC_OK : ERROR_INTERNAL_ERROR;
 }
@@ -912,7 +930,7 @@ static UINT rdpsnd_server_start(RdpsndServerContext* context)
 		goto out_close;
 	}
 
-	CopyMemory(&priv->channelEvent, buffer, sizeof(HANDLE));
+	priv->channelEvent = *(HANDLE*)buffer;
 	WTSFreeMemory(buffer);
 	priv->rdpsnd_pdu = Stream_New(NULL, 4096);
 
@@ -957,7 +975,7 @@ static UINT rdpsnd_server_start(RdpsndServerContext* context)
 
 	return CHANNEL_RC_OK;
 out_stopEvent:
-	CloseHandle(context->priv->StopEvent);
+	(void)CloseHandle(context->priv->StopEvent);
 	context->priv->StopEvent = NULL;
 out_lock:
 	DeleteCriticalSection(&context->priv->lock);
@@ -965,7 +983,7 @@ out_pdu:
 	Stream_Free(context->priv->rdpsnd_pdu, TRUE);
 	context->priv->rdpsnd_pdu = NULL;
 out_close:
-	WTSVirtualChannelClose(context->priv->ChannelHandle);
+	(void)WTSVirtualChannelClose(context->priv->ChannelHandle);
 	context->priv->ChannelHandle = NULL;
 	return error;
 }
@@ -989,7 +1007,7 @@ static UINT rdpsnd_server_stop(RdpsndServerContext* context)
 	{
 		if (context->priv->StopEvent)
 		{
-			SetEvent(context->priv->StopEvent);
+			(void)SetEvent(context->priv->StopEvent);
 
 			if (WaitForSingleObject(context->priv->Thread, INFINITE) == WAIT_FAILED)
 			{
@@ -998,8 +1016,8 @@ static UINT rdpsnd_server_stop(RdpsndServerContext* context)
 				return error;
 			}
 
-			CloseHandle(context->priv->Thread);
-			CloseHandle(context->priv->StopEvent);
+			(void)CloseHandle(context->priv->Thread);
+			(void)CloseHandle(context->priv->StopEvent);
 			context->priv->Thread = NULL;
 			context->priv->StopEvent = NULL;
 		}
@@ -1015,7 +1033,7 @@ static UINT rdpsnd_server_stop(RdpsndServerContext* context)
 
 	if (context->priv->ChannelHandle)
 	{
-		WTSVirtualChannelClose(context->priv->ChannelHandle);
+		(void)WTSVirtualChannelClose(context->priv->ChannelHandle);
 		context->priv->ChannelHandle = NULL;
 	}
 
