@@ -20,6 +20,7 @@
 #include <cassert>
 #include <cstdio>
 #include <cstdlib>
+#include <algorithm>
 
 #include <SDL3/SDL.h>
 #include <SDL3_ttf/SDL_ttf.h>
@@ -41,7 +42,7 @@ static const SDL_Color backgroundcolor = { 0x38, 0x36, 0x35, 0xff };
 
 static const Uint32 hpadding = 10;
 
-SdlWidget::SdlWidget(SDL_Renderer* renderer, const SDL_FRect& rect, bool input)
+SdlWidget::SdlWidget([[maybe_unused]] SDL_Renderer* renderer, const SDL_FRect& rect, bool input)
     : _rect(rect), _input(input)
 {
 	assert(renderer);
@@ -52,7 +53,7 @@ SdlWidget::SdlWidget(SDL_Renderer* renderer, const SDL_FRect& rect, bool input)
 		widget_log_error(-1, "SDLResourceManager::get");
 	else
 	{
-		_font = TTF_OpenFontIO(ops, 1, 64);
+		_font = TTF_OpenFontIO(ops, true, 64);
 		if (!_font)
 			widget_log_error(-1, "TTF_OpenFontRW");
 	}
@@ -81,7 +82,7 @@ SdlWidget::SdlWidget(SdlWidget&& other) noexcept
 SDL_Texture* SdlWidget::render_text(SDL_Renderer* renderer, const std::string& text,
                                     SDL_Color fgcolor, SDL_FRect& src, SDL_FRect& dst)
 {
-	auto surface = TTF_RenderUTF8_Blended(_font, text.c_str(), fgcolor);
+	auto surface = TTF_RenderText_Blended(_font, text.c_str(), 0, fgcolor);
 	if (!surface)
 	{
 		widget_log_error(-1, "TTF_RenderText_Blended");
@@ -96,12 +97,32 @@ SDL_Texture* SdlWidget::render_text(SDL_Renderer* renderer, const std::string& t
 		return nullptr;
 	}
 
+	std::unique_ptr<TTF_TextEngine, decltype(&TTF_DestroySurfaceTextEngine)> engine(
+	    TTF_CreateRendererTextEngine(renderer), TTF_DestroySurfaceTextEngine);
+	if (!engine)
+	{
+		widget_log_error(-1, "TTF_CreateRendererTextEngine");
+		return nullptr;
+	}
+
+	std::unique_ptr<TTF_Text, decltype(&TTF_DestroyText)> txt(
+	    TTF_CreateText(engine.get(), _font, text.c_str(), text.size()), TTF_DestroyText);
+
+	if (!txt)
+	{
+		widget_log_error(-1, "TTF_CreateText");
+		return nullptr;
+	}
 	int w = 0;
 	int h = 0;
-	TTF_SizeUTF8(_font, text.c_str(), &w, &h);
+	if (!TTF_GetTextSize(txt.get(), &w, &h))
+	{
+		widget_log_error(-1, "TTF_GetTextSize");
+		return nullptr;
+	}
 
-	src.w = w;
-	src.h = h;
+	src.w = static_cast<float>(w);
+	src.h = static_cast<float>(h);
 	/* Do some magic:
 	 * - Add padding before and after text
 	 * - if text is too long only show the last elements
@@ -113,8 +134,7 @@ SDL_Texture* SdlWidget::render_text(SDL_Renderer* renderer, const std::string& t
 	const float scale = dst.h / src.h;
 	const float sws = (src.w) * scale;
 	const float dws = (dst.w) / scale;
-	if (dst.w > sws)
-		dst.w = sws;
+	dst.w = std::min(dst.w, sws);
 	if (src.w > dws)
 	{
 		src.x = src.w - dws;
@@ -123,21 +143,28 @@ SDL_Texture* SdlWidget::render_text(SDL_Renderer* renderer, const std::string& t
 	return texture;
 }
 
+static float scale(float dw, float dh)
+{
+	const auto scale = dh / dw;
+	const auto dr = dh * scale;
+	return dr;
+}
+
 SDL_Texture* SdlWidget::render_text_wrapped(SDL_Renderer* renderer, const std::string& text,
                                             SDL_Color fgcolor, SDL_FRect& src, SDL_FRect& dst)
 {
-	Sint32 w = 0;
-	Sint32 h = 0;
-	TTF_SizeUTF8(_font, " ", &w, &h);
-	auto surface = TTF_RenderUTF8_Blended_Wrapped(_font, text.c_str(), fgcolor, _text_width);
+	assert(_text_width < INT32_MAX);
+
+	auto surface = TTF_RenderText_Blended_Wrapped(_font, text.c_str(), 0, fgcolor,
+	                                              static_cast<int>(_text_width));
 	if (!surface)
 	{
 		widget_log_error(-1, "TTF_RenderText_Blended");
 		return nullptr;
 	}
 
-	src.w = surface->w;
-	src.h = surface->h;
+	src.w = static_cast<float>(surface->w);
+	src.h = static_cast<float>(surface->h);
 
 	auto texture = SDL_CreateTextureFromSurface(renderer, surface);
 	SDL_DestroySurface(surface);
@@ -155,10 +182,8 @@ SDL_Texture* SdlWidget::render_text_wrapped(SDL_Renderer* renderer, const std::s
 	dst = _rect;
 	dst.x += hpadding;
 	dst.w -= 2 * hpadding;
-	const float scale = static_cast<float>(src.h) / static_cast<float>(src.w);
-	auto dh = src.h * scale;
-	if (dh < dst.h)
-		dst.h = dh;
+	auto dh = scale(src.w, src.h);
+	dst.h = std::min<float>(dh, dst.h);
 
 	return texture;
 }
@@ -170,13 +195,20 @@ SdlWidget::~SdlWidget()
 		SDL_DestroyTexture(_image);
 }
 
-bool SdlWidget::error_ex(Uint32 res, const char* what, const char* file, size_t line,
+bool SdlWidget::error_ex(bool success, const char* what, const char* file, size_t line,
                          const char* fkt)
 {
+	if (success)
+	{
+		// Flip SDL3 convention to existing code convention to minimize code changes
+		return false;
+	}
 	static wLog* log = nullptr;
 	if (!log)
 		log = WLog_Get(TAG);
-	return sdl_log_error_ex(res, log, what, file, line, fkt);
+	// Use -1 as it indicates error similar to SDL2 conventions
+	// sdl_log_error_ex treats any value other than 0 as SDL error
+	return sdl_log_error_ex(-1, log, what, file, line, fkt);
 }
 
 static bool draw_rect(SDL_Renderer* renderer, const SDL_FRect* rect, SDL_Color color)
@@ -252,12 +284,12 @@ bool SdlWidget::update_text(SDL_Renderer* renderer, const std::string& text, SDL
 		texture = _image;
 		dst = _rect;
 		auto propId = SDL_GetTextureProperties(_image);
-		int w = SDL_GetNumberProperty(propId, SDL_PROP_TEXTURE_WIDTH_NUMBER, -1);
-		int h = SDL_GetNumberProperty(propId, SDL_PROP_TEXTURE_HEIGHT_NUMBER, -1);
+		auto w = SDL_GetNumberProperty(propId, SDL_PROP_TEXTURE_WIDTH_NUMBER, -1);
+		auto h = SDL_GetNumberProperty(propId, SDL_PROP_TEXTURE_HEIGHT_NUMBER, -1);
 		if (w < 0 || h < 0)
 			widget_log_error(-1, "SDL_GetTextureProperties");
-		src.w = w;
-		src.h = h;
+		src.w = static_cast<float>(w);
+		src.h = static_cast<float>(h);
 	}
 	else if (_wrap)
 		texture = render_text_wrapped(renderer, text, fgcolor, src, dst);
