@@ -41,6 +41,7 @@
 #include "xf_graphics.h"
 #include "xf_utils.h"
 
+#include "xf_debug.h"
 #include "xf_event.h"
 
 #define TAG CLIENT_TAG("x11")
@@ -165,15 +166,6 @@ const char* x11_event_string(int event)
 	}
 }
 
-#ifdef WITH_DEBUG_X11
-#define DEBUG_X11(...) WLog_DBG(TAG, __VA_ARGS__)
-#else
-#define DEBUG_X11(...) \
-	do                 \
-	{                  \
-	} while (0)
-#endif
-
 static BOOL xf_action_script_append(xfContext* xfc, const char* buffer, size_t size,
                                     WINPR_ATTR_UNUSED void* user, const char* what, const char* arg)
 {
@@ -1216,7 +1208,24 @@ BOOL xf_event_process(freerdp* instance, const XEvent* event)
 		}
 
 		if (xf_floatbar_is_locked(floatbar))
-			return TRUE;
+		{
+			/* Filter input events, floatbar is locked do not forward anything to the session */
+			switch (event->type)
+			{
+				case MotionNotify:
+				case ButtonPress:
+				case ButtonRelease:
+				case KeyPress:
+				case KeyRelease:
+				case FocusIn:
+				case FocusOut:
+				case EnterNotify:
+				case LeaveNotify:
+					return TRUE;
+				default:
+					break;
+			}
+		}
 	}
 
 	xf_event_execute_action_script(xfc, event);
