@@ -793,10 +793,13 @@ static BOOL clear_decompress_bands_data(CLEAR_CONTEXT* WINPR_RESTRICT clear,
 				if ((y + count) > vBarPixelCount)
 					count = (vBarPixelCount > y) ? (vBarPixelCount - y) : 0;
 
-				while (count--)
+				if (count > 0)
 				{
-					FreeRDPWriteColor(dstBuffer, clear->format, colorBkg);
-					dstBuffer += FreeRDPGetBytesPerPixel(clear->format);
+					while (count--)
+					{
+						FreeRDPWriteColor(dstBuffer, clear->format, colorBkg);
+						dstBuffer += FreeRDPGetBytesPerPixel(clear->format);
+					}
 				}
 
 				/*
@@ -836,12 +839,15 @@ static BOOL clear_decompress_bands_data(CLEAR_CONTEXT* WINPR_RESTRICT clear,
 				y = vBarYOn + vBarShortPixelCount;
 				count = (vBarPixelCount > y) ? (vBarPixelCount - y) : 0;
 
-				while (count--)
+				if (count > 0)
 				{
-					if (!FreeRDPWriteColor(dstBuffer, clear->format, colorBkg))
-						return FALSE;
+					while (count--)
+					{
+						if (!FreeRDPWriteColor(dstBuffer, clear->format, colorBkg))
+							return FALSE;
 
-					dstBuffer += FreeRDPGetBytesPerPixel(clear->format);
+						dstBuffer += FreeRDPGetBytesPerPixel(clear->format);
+					}
 				}
 
 				vBarEntry->count = vBarPixelCount;
@@ -1144,8 +1150,11 @@ fail:
 	return rc;
 }
 
-int clear_compress(CLEAR_CONTEXT* WINPR_RESTRICT clear, const BYTE* WINPR_RESTRICT pSrcData,
-                   UINT32 SrcSize, BYTE** WINPR_RESTRICT ppDstData, UINT32* WINPR_RESTRICT pDstSize)
+int clear_compress(WINPR_ATTR_UNUSED CLEAR_CONTEXT* WINPR_RESTRICT clear,
+                   WINPR_ATTR_UNUSED const BYTE* WINPR_RESTRICT pSrcData,
+                   WINPR_ATTR_UNUSED UINT32 SrcSize,
+                   WINPR_ATTR_UNUSED BYTE** WINPR_RESTRICT ppDstData,
+                   WINPR_ATTR_UNUSED UINT32* WINPR_RESTRICT pDstSize)
 {
 	WLog_ERR(TAG, "TODO: not implemented!");
 	return 1;
