@@ -87,7 +87,8 @@ int region16_n_rects(const REGION16* region)
 {
 	WINPR_ASSERT(region);
 	WINPR_ASSERT(region->data);
-	return region->data->nbRects;
+	WINPR_ASSERT(region->data->nbRects <= INT32_MAX);
+	return (int)region->data->nbRects;
 }
 
 const RECTANGLE_16* region16_rects(const REGION16* region, UINT32* nbRects)
@@ -106,7 +107,10 @@ const RECTANGLE_16* region16_rects(const REGION16* region, UINT32* nbRects)
 		return NULL;
 
 	if (nbRects)
-		*nbRects = data->nbRects;
+	{
+		WINPR_ASSERT(data->nbRects <= UINT32_MAX);
+		*nbRects = (UINT32)data->nbRects;
+	}
 
 	return (RECTANGLE_16*)(data + 1);
 }
@@ -190,13 +194,14 @@ void region16_clear(REGION16* region)
 
 static INLINE REGION16_DATA* allocateRegion(long nbItems)
 {
-	long allocSize = sizeof(REGION16_DATA) + (nbItems * sizeof(RECTANGLE_16));
+	size_t allocSize =
+	    sizeof(REGION16_DATA) + (WINPR_ASSERTING_INT_CAST(size_t, nbItems) * sizeof(RECTANGLE_16));
 	REGION16_DATA* ret = (REGION16_DATA*)malloc(allocSize);
 
 	if (!ret)
 		return ret;
 
-	ret->size = allocSize;
+	ret->size = WINPR_ASSERTING_INT_CAST(long, allocSize);
 	ret->nbRects = nbItems;
 	return ret;
 }
@@ -225,7 +230,7 @@ BOOL region16_copy(REGION16* dst, const REGION16* src)
 		if (!dst->data)
 			return FALSE;
 
-		CopyMemory(dst->data, src->data, src->data->size);
+		CopyMemory(dst->data, src->data, WINPR_ASSERTING_INT_CAST(size_t, src->data->size));
 	}
 
 	return TRUE;
@@ -278,7 +283,7 @@ static void region16_copy_band_with_union(RECTANGLE_16* dst, const RECTANGLE_16*
 	 *
 	 *  We first copy as-is items that are before Item2, the first overlapping
 	 *  item.
-	 *  Then we find the last one that overlap unionRect to agregate Item2, Item3
+	 *  Then we find the last one that overlap unionRect to aggregate Item2, Item3
 	 *  and Item4 to create Item2.
 	 *  Finally Item5 is copied as Item3.
 	 *
@@ -423,7 +428,6 @@ static BOOL region16_simplify_bands(REGION16* region)
 	int nbRects = 0;
 	int finalNbRects = 0;
 	int bandItems = 0;
-	int toMove = 0;
 	finalNbRects = nbRects = region16_n_rects(region);
 
 	if (nbRects < 2)
@@ -453,7 +457,8 @@ static BOOL region16_simplify_bands(REGION16* region)
 			/* override band2, we don't move band1 pointer as the band after band2
 			 * may be merged too */
 			endBand = band2 + bandItems;
-			toMove = (endPtr - endBand) * sizeof(RECTANGLE_16);
+			const size_t toMove =
+			    WINPR_ASSERTING_INT_CAST(size_t, (endPtr - endBand)) * sizeof(RECTANGLE_16);
 
 			if (toMove)
 				MoveMemory(band2, endBand, toMove);
@@ -469,7 +474,8 @@ static BOOL region16_simplify_bands(REGION16* region)
 
 	if (finalNbRects != nbRects)
 	{
-		size_t allocSize = sizeof(REGION16_DATA) + (finalNbRects * sizeof(RECTANGLE_16));
+		size_t allocSize = sizeof(REGION16_DATA) +
+		                   (WINPR_ASSERTING_INT_CAST(size_t, finalNbRects) * sizeof(RECTANGLE_16));
 		REGION16_DATA* data = realloc(region->data, allocSize);
 		if (!data)
 			free(region->data);
@@ -482,7 +488,7 @@ static BOOL region16_simplify_bands(REGION16* region)
 		}
 
 		region->data->nbRects = finalNbRects;
-		region->data->size = allocSize;
+		region->data->size = WINPR_ASSERTING_INT_CAST(long, allocSize);
 	}
 
 	return TRUE;
@@ -523,7 +529,7 @@ BOOL region16_union_rect(REGION16* dst, const REGION16* src, const RECTANGLE_16*
 		return TRUE;
 	}
 
-	newItems = allocateRegion((1ULL + 4ULL * region16_n_rects(src)));
+	newItems = allocateRegion((1L + 4L * region16_n_rects(src)));
 
 	if (!newItems)
 		return FALSE;
@@ -586,13 +592,13 @@ BOOL region16_union_rect(REGION16* dst, const REGION16* src, const RECTANGLE_16*
 			           |    |  |    |
 
 			 possible cases:
-			 1) no top split, merge zone then a bottom split. The band will be splitted
+			 1) no top split, merge zone then a bottom split. The band will be split
 			  in two
-			 2) not band split, only the merge zone, band merged with rect but not splitted
+			 2) not band split, only the merge zone, band merged with rect but not split
 			 3) a top split, the merge zone and no bottom split. The band will be split
 			 in two
 			 4) a top split, the merge zone and also a bottom split. The band will be
-			 splitted in 3, but the coalesce algorithm may merge the created bands
+			 split in 3, but the coalesce algorithm may merge the created bands
 			 */
 			UINT16 mergeTop = currentBand->top;
 			UINT16 mergeBottom = currentBand->bottom;
@@ -670,10 +676,13 @@ BOOL region16_union_rect(REGION16* dst, const REGION16* src, const RECTANGLE_16*
 	dstExtents->left = MIN(rect->left, srcExtents->left);
 	dstExtents->bottom = MAX(rect->bottom, srcExtents->bottom);
 	dstExtents->right = MAX(rect->right, srcExtents->right);
-	newItems->size = sizeof(REGION16_DATA) + (usedRects * sizeof(RECTANGLE_16));
-	tmpItems = realloc(newItems, newItems->size);
+	newItems->size =
+	    WINPR_ASSERTING_INT_CAST(long, sizeof(REGION16_DATA) + (usedRects * sizeof(RECTANGLE_16)));
+	if (newItems->size != 0)
+		tmpItems = realloc(newItems, WINPR_ASSERTING_INT_CAST(size_t, newItems->size));
 	if (!tmpItems)
 		free(newItems);
+
 	newItems = tmpItems;
 	dst->data = newItems;
 
@@ -789,12 +798,13 @@ BOOL region16_intersect_rect(REGION16* dst, const REGION16* src, const RECTANGLE
 	}
 
 	newItems->nbRects = usedRects;
-	newItems->size = sizeof(REGION16_DATA) + (usedRects * sizeof(RECTANGLE_16));
+	newItems->size =
+	    WINPR_ASSERTING_INT_CAST(long, sizeof(REGION16_DATA) + (usedRects * sizeof(RECTANGLE_16)));
 
 	if ((dst->data->size > 0) && (dst->data != &empty_region))
 		free(dst->data);
 
-	dst->data = realloc(newItems, newItems->size);
+	dst->data = realloc(newItems, WINPR_ASSERTING_INT_CAST(size_t, newItems->size));
 
 	if (!dst->data)
 	{
