@@ -24,17 +24,15 @@
 
 static BOOL compareRectangles(const RECTANGLE_16* src1, const RECTANGLE_16* src2, int nb)
 {
-	int i;
-
-	for (i = 0; i < nb; i++, src1++, src2++)
+	for (int i = 0; i < nb; i++, src1++, src2++)
 	{
-		if (memcmp(src1, src2, sizeof(RECTANGLE_16)))
+		if (memcmp(src1, src2, sizeof(RECTANGLE_16)) != 0)
 		{
-			fprintf(stderr,
-			        "expecting rect %d (%" PRIu16 ",%" PRIu16 "-%" PRIu16 ",%" PRIu16
-			        ") and have (%" PRIu16 ",%" PRIu16 "-%" PRIu16 ",%" PRIu16 ")\n",
-			        i, src2->left, src2->top, src2->right, src2->bottom, src1->left, src1->top,
-			        src1->right, src1->bottom);
+			(void)fprintf(stderr,
+			              "expecting rect %d (%" PRIu16 ",%" PRIu16 "-%" PRIu16 ",%" PRIu16
+			              ") and have (%" PRIu16 ",%" PRIu16 "-%" PRIu16 ",%" PRIu16 ")\n",
+			              i, src2->left, src2->top, src2->right, src2->bottom, src1->left,
+			              src1->top, src1->right, src1->bottom);
 			return FALSE;
 		}
 	}
@@ -46,8 +44,8 @@ static int test_basic(void)
 {
 	REGION16 region;
 	int retCode = -1;
-	const RECTANGLE_16* rects;
-	UINT32 nbRects;
+	const RECTANGLE_16* rects = NULL;
+	UINT32 nbRects = 0;
 	/* R1 + R2 ==> disjointed rects */
 	RECTANGLE_16 r1 = { 0, 101, 200, 201 };
 	RECTANGLE_16 r2 = { 150, 301, 250, 401 };
@@ -60,7 +58,7 @@ static int test_basic(void)
 
 	rects = region16_rects(&region, &nbRects);
 
-	if (!rects || nbRects != 1 || memcmp(rects, &r1, sizeof(RECTANGLE_16)))
+	if (!rects || nbRects != 1 || memcmp(rects, &r1, sizeof(RECTANGLE_16)) != 0)
 		goto out;
 
 	/* r1 + r2 */
@@ -69,7 +67,8 @@ static int test_basic(void)
 
 	rects = region16_rects(&region, &nbRects);
 
-	if (!rects || nbRects != 2 || !compareRectangles(rects, r1_r2, nbRects))
+	if (!rects || nbRects != 2 ||
+	    !compareRectangles(rects, r1_r2, WINPR_ASSERTING_INT_CAST(int, nbRects)))
 		goto out;
 
 	/* clear region */
@@ -89,8 +88,8 @@ static int test_r1_r3(void)
 {
 	REGION16 region;
 	int retCode = -1;
-	const RECTANGLE_16* rects;
-	UINT32 nbRects;
+	const RECTANGLE_16* rects = NULL;
+	UINT32 nbRects = 0;
 	RECTANGLE_16 r1 = { 0, 101, 200, 201 };
 	RECTANGLE_16 r3 = { 150, 151, 250, 251 };
 	RECTANGLE_16 r1_r3[] = { { 0, 101, 200, 151 }, { 0, 151, 250, 201 }, { 150, 201, 250, 251 } };
@@ -116,7 +115,8 @@ static int test_r1_r3(void)
 
 	rects = region16_rects(&region, &nbRects);
 
-	if (!rects || nbRects != 3 || !compareRectangles(rects, r1_r3, nbRects))
+	if (!rects || nbRects != 3 ||
+	    !compareRectangles(rects, r1_r3, WINPR_ASSERTING_INT_CAST(int, nbRects)))
 		goto out;
 
 	/* R3 + R1  */
@@ -130,7 +130,8 @@ static int test_r1_r3(void)
 
 	rects = region16_rects(&region, &nbRects);
 
-	if (!rects || nbRects != 3 || !compareRectangles(rects, r1_r3, nbRects))
+	if (!rects || nbRects != 3 ||
+	    !compareRectangles(rects, r1_r3, WINPR_ASSERTING_INT_CAST(int, nbRects)))
 		goto out;
 
 	retCode = 0;
@@ -143,8 +144,8 @@ static int test_r9_r10(void)
 {
 	REGION16 region;
 	int retCode = -1;
-	const RECTANGLE_16* rects;
-	UINT32 nbRects;
+	const RECTANGLE_16* rects = NULL;
+	UINT32 nbRects = 0;
 	/*
 	 * +===============================================================
 	 * |
@@ -175,7 +176,8 @@ static int test_r9_r10(void)
 
 	rects = region16_rects(&region, &nbRects);
 
-	if (!rects || nbRects != 3 || !compareRectangles(rects, r9_r10, nbRects))
+	if (!rects || nbRects != 3 ||
+	    !compareRectangles(rects, r9_r10, WINPR_ASSERTING_INT_CAST(int, nbRects)))
 		goto out;
 
 	retCode = 0;
@@ -188,8 +190,8 @@ static int test_r1_r5(void)
 {
 	REGION16 region;
 	int retCode = -1;
-	const RECTANGLE_16* rects;
-	UINT32 nbRects;
+	const RECTANGLE_16* rects = NULL;
+	UINT32 nbRects = 0;
 	RECTANGLE_16 r1 = { 0, 101, 200, 201 };
 	RECTANGLE_16 r5 = { 150, 121, 300, 131 };
 	RECTANGLE_16 r1_r5[] = { { 0, 101, 200, 121 }, { 0, 121, 300, 131 }, { 0, 131, 200, 201 } };
@@ -216,7 +218,8 @@ static int test_r1_r5(void)
 
 	rects = region16_rects(&region, &nbRects);
 
-	if (!rects || nbRects != 3 || !compareRectangles(rects, r1_r5, nbRects))
+	if (!rects || nbRects != 3 ||
+	    !compareRectangles(rects, r1_r5, WINPR_ASSERTING_INT_CAST(int, nbRects)))
 		goto out;
 
 	retCode = 0;
@@ -229,8 +232,8 @@ static int test_r1_r6(void)
 {
 	REGION16 region;
 	int retCode = -1;
-	const RECTANGLE_16* rects;
-	UINT32 nbRects;
+	const RECTANGLE_16* rects = NULL;
+	UINT32 nbRects = 0;
 	RECTANGLE_16 r1 = { 0, 101, 200, 201 };
 	RECTANGLE_16 r6 = { 150, 121, 170, 131 };
 	region16_init(&region);
@@ -256,7 +259,8 @@ static int test_r1_r6(void)
 
 	rects = region16_rects(&region, &nbRects);
 
-	if (!rects || nbRects != 1 || !compareRectangles(rects, &r1, nbRects))
+	if (!rects || nbRects != 1 ||
+	    !compareRectangles(rects, &r1, WINPR_ASSERTING_INT_CAST(int, nbRects)))
 		goto out;
 
 	retCode = 0;
@@ -269,8 +273,8 @@ static int test_r1_r2_r4(void)
 {
 	REGION16 region;
 	int retCode = -1;
-	const RECTANGLE_16* rects;
-	UINT32 nbRects;
+	const RECTANGLE_16* rects = NULL;
+	UINT32 nbRects = 0;
 	RECTANGLE_16 r1 = { 0, 101, 200, 201 };
 	RECTANGLE_16 r2 = { 150, 301, 250, 401 };
 	RECTANGLE_16 r4 = { 150, 251, 250, 301 };
@@ -305,7 +309,8 @@ static int test_r1_r2_r4(void)
 
 	rects = region16_rects(&region, &nbRects);
 
-	if (!rects || nbRects != 2 || !compareRectangles(rects, r1_r2_r4, nbRects))
+	if (!rects || nbRects != 2 ||
+	    !compareRectangles(rects, r1_r2_r4, WINPR_ASSERTING_INT_CAST(int, nbRects)))
 		goto out;
 
 	retCode = 0;
@@ -318,8 +323,8 @@ static int test_r1_r7_r8(void)
 {
 	REGION16 region;
 	int retCode = -1;
-	const RECTANGLE_16* rects;
-	UINT32 nbRects;
+	const RECTANGLE_16* rects = NULL;
+	UINT32 nbRects = 0;
 	RECTANGLE_16 r1 = { 0, 101, 200, 201 };
 	RECTANGLE_16 r7 = { 300, 101, 500, 201 };
 	RECTANGLE_16 r8 = { 150, 121, 400, 131 };
@@ -352,7 +357,8 @@ static int test_r1_r7_r8(void)
 
 	rects = region16_rects(&region, &nbRects);
 
-	if (!rects || nbRects != 5 || !compareRectangles(rects, r1_r7_r8, nbRects))
+	if (!rects || nbRects != 5 ||
+	    !compareRectangles(rects, r1_r7_r8, WINPR_ASSERTING_INT_CAST(int, nbRects)))
 		goto out;
 
 	region16_clear(&region);
@@ -368,7 +374,8 @@ static int test_r1_r7_r8(void)
 
 	rects = region16_rects(&region, &nbRects);
 
-	if (!rects || nbRects != 5 || !compareRectangles(rects, r1_r7_r8, nbRects))
+	if (!rects || nbRects != 5 ||
+	    !compareRectangles(rects, r1_r7_r8, WINPR_ASSERTING_INT_CAST(int, nbRects)))
 		goto out;
 
 	region16_clear(&region);
@@ -384,7 +391,8 @@ static int test_r1_r7_r8(void)
 
 	rects = region16_rects(&region, &nbRects);
 
-	if (!rects || nbRects != 5 || !compareRectangles(rects, r1_r7_r8, nbRects))
+	if (!rects || nbRects != 5 ||
+	    !compareRectangles(rects, r1_r7_r8, WINPR_ASSERTING_INT_CAST(int, nbRects)))
 		goto out;
 
 	retCode = 0;
@@ -397,8 +405,8 @@ static int test_r1_r2_r3_r4(void)
 {
 	REGION16 region;
 	int retCode = -1;
-	const RECTANGLE_16* rects;
-	UINT32 nbRects;
+	const RECTANGLE_16* rects = NULL;
+	UINT32 nbRects = 0;
 	RECTANGLE_16 r1 = { 0, 101, 200, 201 };
 	RECTANGLE_16 r2 = { 150, 301, 250, 401 };
 	RECTANGLE_16 r3 = { 150, 151, 250, 251 };
@@ -479,8 +487,8 @@ static int test_from_weston(void)
 	 */
 	REGION16 region;
 	int retCode = -1;
-	const RECTANGLE_16* rects;
-	UINT32 nbRects;
+	const RECTANGLE_16* rects = NULL;
+	UINT32 nbRects = 0;
 	RECTANGLE_16 r1 = { 0, 0, 640, 32 };
 	RECTANGLE_16 r2 = { 236, 169, 268, 201 };
 	RECTANGLE_16 r3 = { 246, 258, 278, 290 };
@@ -524,10 +532,11 @@ out:
 
 static int test_r1_inter_r3(void)
 {
-	REGION16 region, intersection;
+	REGION16 region;
+	REGION16 intersection;
 	int retCode = -1;
-	const RECTANGLE_16* rects;
-	UINT32 nbRects;
+	const RECTANGLE_16* rects = NULL;
+	UINT32 nbRects = 0;
 	RECTANGLE_16 r1 = { 0, 101, 200, 201 };
 	RECTANGLE_16 r3 = { 150, 151, 250, 251 };
 	RECTANGLE_16 r1_inter_r3[] = {
@@ -558,7 +567,8 @@ static int test_r1_inter_r3(void)
 
 	rects = region16_rects(&intersection, &nbRects);
 
-	if (!rects || nbRects != 1 || !compareRectangles(rects, r1_inter_r3, nbRects))
+	if (!rects || nbRects != 1 ||
+	    !compareRectangles(rects, r1_inter_r3, WINPR_ASSERTING_INT_CAST(int, nbRects)))
 		goto out;
 
 	retCode = 0;
@@ -570,10 +580,11 @@ out:
 
 static int test_r1_r3_inter_r11(void)
 {
-	REGION16 region, intersection;
+	REGION16 region;
+	REGION16 intersection;
 	int retCode = -1;
-	const RECTANGLE_16* rects;
-	UINT32 nbRects;
+	const RECTANGLE_16* rects = NULL;
+	UINT32 nbRects = 0;
 	RECTANGLE_16 r1 = { 0, 101, 200, 201 };
 	RECTANGLE_16 r3 = { 150, 151, 250, 251 };
 	RECTANGLE_16 r11 = { 170, 151, 600, 301 };
@@ -617,7 +628,8 @@ static int test_r1_r3_inter_r11(void)
 
 	rects = region16_rects(&intersection, &nbRects);
 
-	if (!rects || nbRects != 1 || !compareRectangles(rects, r1_r3_inter_r11, nbRects))
+	if (!rects || nbRects != 1 ||
+	    !compareRectangles(rects, r1_r3_inter_r11, WINPR_ASSERTING_INT_CAST(int, nbRects)))
 		goto out;
 
 	retCode = 0;
@@ -629,10 +641,11 @@ out:
 
 static int test_norbert_case(void)
 {
-	REGION16 region, intersection;
+	REGION16 region;
+	REGION16 intersection;
 	int retCode = -1;
-	const RECTANGLE_16* rects;
-	UINT32 nbRects, i;
+	const RECTANGLE_16* rects = NULL;
+	UINT32 nbRects = 0;
 	RECTANGLE_16 inRectangles[5] = { { 1680, 0, 1920, 242 },
 		                             { 294, 242, 971, 776 },
 		                             { 1680, 242, 1920, 776 },
@@ -669,7 +682,7 @@ static int test_norbert_case(void)
 	 * After intersected with the full screen rect, the
 	 * result should keep the same.
 	 */
-	for (i = 0; i < 5; i++)
+	for (int i = 0; i < 5; i++)
 	{
 		if (!region16_union_rect(&region, &region, &inRectangles[i]))
 			goto out;
@@ -683,7 +696,8 @@ static int test_norbert_case(void)
 
 	rects = region16_rects(&intersection, &nbRects);
 
-	if (!rects || nbRects != 5 || !compareRectangles(rects, inRectangles, nbRects))
+	if (!rects || nbRects != 5 ||
+	    !compareRectangles(rects, inRectangles, WINPR_ASSERTING_INT_CAST(int, nbRects)))
 		goto out;
 
 	if (!compareRectangles(region16_extents(&intersection), &expected_inter_extents, 1))
@@ -700,7 +714,7 @@ static int test_norbert2_case(void)
 {
 	REGION16 region;
 	int retCode = -1;
-	const RECTANGLE_16* rects;
+	const RECTANGLE_16* rects = NULL;
 	UINT32 nbRects = 0;
 	RECTANGLE_16 rect1 = { 464, 696, 476, 709 };
 	RECTANGLE_16 rect2 = { 0, 0, 1024, 32 };
@@ -708,57 +722,57 @@ static int test_norbert2_case(void)
 
 	if (!region16_union_rect(&region, &region, &rect1))
 	{
-		fprintf(stderr, "%s: Error 1 - region16_union_rect failed\n", __func__);
+		(void)fprintf(stderr, "%s: Error 1 - region16_union_rect failed\n", __func__);
 		goto out;
 	}
 
 	if (!(rects = region16_rects(&region, &nbRects)))
 	{
-		fprintf(stderr, "%s: Error 2 - region16_rects failed\n", __func__);
+		(void)fprintf(stderr, "%s: Error 2 - region16_rects failed\n", __func__);
 		goto out;
 	}
 
 	if (nbRects != 1)
 	{
-		fprintf(stderr, "%s: Error 3 - expected nbRects == 1 but got %" PRIu32 "\n", __func__,
-		        nbRects);
+		(void)fprintf(stderr, "%s: Error 3 - expected nbRects == 1 but got %" PRIu32 "\n", __func__,
+		              nbRects);
 		goto out;
 	}
 
 	if (!compareRectangles(&rects[0], &rect1, 1))
 	{
-		fprintf(stderr, "%s: Error 4 - compare failed\n", __func__);
+		(void)fprintf(stderr, "%s: Error 4 - compare failed\n", __func__);
 		goto out;
 	}
 
 	if (!region16_union_rect(&region, &region, &rect2))
 	{
-		fprintf(stderr, "%s: Error 5 - region16_union_rect failed\n", __func__);
+		(void)fprintf(stderr, "%s: Error 5 - region16_union_rect failed\n", __func__);
 		goto out;
 	}
 
 	if (!(rects = region16_rects(&region, &nbRects)))
 	{
-		fprintf(stderr, "%s: Error 6 - region16_rects failed\n", __func__);
+		(void)fprintf(stderr, "%s: Error 6 - region16_rects failed\n", __func__);
 		goto out;
 	}
 
 	if (nbRects != 2)
 	{
-		fprintf(stderr, "%s: Error 7 - expected nbRects == 2 but got %" PRIu32 "\n", __func__,
-		        nbRects);
+		(void)fprintf(stderr, "%s: Error 7 - expected nbRects == 2 but got %" PRIu32 "\n", __func__,
+		              nbRects);
 		goto out;
 	}
 
 	if (!compareRectangles(&rects[0], &rect2, 1))
 	{
-		fprintf(stderr, "%s: Error 8 - compare failed\n", __func__);
+		(void)fprintf(stderr, "%s: Error 8 - compare failed\n", __func__);
 		goto out;
 	}
 
 	if (!compareRectangles(&rects[1], &rect1, 1))
 	{
-		fprintf(stderr, "%s: Error 9 - compare failed\n", __func__);
+		(void)fprintf(stderr, "%s: Error 9 - compare failed\n", __func__);
 		goto out;
 	}
 
@@ -770,9 +784,9 @@ out:
 
 static int test_empty_rectangle(void)
 {
-	REGION16 region, intersection;
+	REGION16 region;
+	REGION16 intersection;
 	int retCode = -1;
-	int i;
 	RECTANGLE_16 emptyRectangles[3] = { { 0, 0, 0, 0 }, { 10, 10, 10, 11 }, { 10, 10, 11, 10 } };
 	RECTANGLE_16 firstRect = { 0, 0, 100, 100 };
 	RECTANGLE_16 anotherRect = { 100, 100, 200, 200 };
@@ -781,7 +795,7 @@ static int test_empty_rectangle(void)
 	region16_init(&intersection);
 
 	/* Check for empty rectangles */
-	for (i = 0; i < 3; i++)
+	for (int i = 0; i < 3; i++)
 	{
 		if (!rectangle_is_empty(&emptyRectangles[i]))
 			goto out;
@@ -840,15 +854,15 @@ static struct UnitaryTest tests[] = { { "Basic trivial tests", test_basic },
 
 int TestFreeRDPRegion(int argc, char* argv[])
 {
-	int i, testNb = 0;
+	int testNb = 0;
 	int retCode = -1;
 	WINPR_UNUSED(argc);
 	WINPR_UNUSED(argv);
 
-	for (i = 0; tests[i].func; i++)
+	for (int i = 0; tests[i].func; i++)
 	{
 		testNb++;
-		fprintf(stderr, "%d: %s\n", testNb, tests[i].name);
+		(void)fprintf(stderr, "%d: %s\n", testNb, tests[i].name);
 		retCode = tests[i].func();
 
 		if (retCode < 0)
@@ -856,7 +870,7 @@ int TestFreeRDPRegion(int argc, char* argv[])
 	}
 
 	if (retCode < 0)
-		fprintf(stderr, "failed for test %d\n", testNb);
+		(void)fprintf(stderr, "failed for test %d\n", testNb);
 
 	return retCode;
 }
