@@ -20,6 +20,7 @@
 #include <freerdp/config.h>
 
 #include <errno.h>
+#include <stdint.h>
 
 #include <winpr/crt.h>
 #include <winpr/print.h>
@@ -83,7 +84,7 @@ struct s_http_response
 	size_t count;
 	char** lines;
 
-	long StatusCode;
+	INT16 StatusCode;
 	const char* ReasonPhrase;
 
 	size_t ContentLength;
@@ -345,7 +346,7 @@ BOOL http_context_set_pragma(HttpContext* context, WINPR_FORMAT_ARG const char*
 	free(context->Pragma);
 	context->Pragma = NULL;
 
-	va_list ap;
+	va_list ap = { 0 };
 	va_start(ap, Pragma);
 	return list_append(context, Pragma, ap);
 }
@@ -356,7 +357,7 @@ BOOL http_context_append_pragma(HttpContext* context, const char* Pragma, ...)
 	if (!context || !Pragma)
 		return FALSE;
 
-	va_list ap;
+	va_list ap = { 0 };
 	va_start(ap, Pragma);
 	return list_append(context, Pragma, ap);
 }
@@ -551,7 +552,7 @@ WINPR_ATTR_FORMAT_ARG(2, 3)
 static BOOL http_encode_print(wStream* s, WINPR_FORMAT_ARG const char* fmt, ...)
 {
 	char* str = NULL;
-	va_list ap;
+	va_list ap = { 0 };
 	int length = 0;
 	int used = 0;
 
@@ -588,7 +589,7 @@ static BOOL http_encode_body_line(wStream* s, const char* param, const char* val
 
 static BOOL http_encode_content_length_line(wStream* s, size_t ContentLength)
 {
-	return http_encode_print(s, "Content-Length: %" PRIdz "\r\n", ContentLength);
+	return http_encode_print(s, "Content-Length: %" PRIuz "\r\n", ContentLength);
 }
 
 static BOOL http_encode_header_line(wStream* s, const char* Method, const char* URI)
@@ -812,7 +813,7 @@ static BOOL http_response_parse_header_status_line(HttpResponse* response, const
 		if ((errno != 0) || (val < 0) || (val > INT16_MAX))
 			goto fail;
 
-		response->StatusCode = strtol(status_code, NULL, 0);
+		response->StatusCode = (INT16)val;
 	}
 	response->ReasonPhrase = reason_phrase;
 
@@ -1030,10 +1031,10 @@ static BOOL http_response_parse_header(HttpResponse* response)
 				break;
 		}
 
-		if (!http_response_parse_header_field(response, name, value))
-			goto fail;
-
+		const int res = http_response_parse_header_field(response, name, value);
 		*end_of_header = end_of_header_char;
+		if (!res)
+			goto fail;
 	}
 
 	rc = TRUE;
@@ -1110,10 +1111,11 @@ static int print_bio_error(const char* str, size_t len, void* bp)
 {
 	wLog* log = bp;
 
-	WINPR_UNUSED(len);
 	WINPR_UNUSED(bp);
 	WLog_Print(log, WLOG_ERROR, "%s", str);
-	return len;
+	if (len > INT32_MAX)
+		return -1;
+	return (int)len;
 }
 
 int http_chuncked_read(BIO* bio, BYTE* pBuffer, size_t size,
@@ -1124,20 +1126,24 @@ int http_chuncked_read(BIO* bio, BYTE* pBuffer, size_t size,
 	WINPR_ASSERT(bio);
 	WINPR_ASSERT(pBuffer);
 	WINPR_ASSERT(encodingContext != NULL);
+	WINPR_ASSERT(size <= INT32_MAX);
 	while (TRUE)
 	{
 		switch (encodingContext->state)
 		{
 			case ChunkStateData:
 			{
+				const size_t rd =
+				    (size > encodingContext->nextOffset ? encodingContext->nextOffset : size);
+				if (rd > INT32_MAX)
+					return -1;
+
 				ERR_clear_error();
-				status = BIO_read(
-				    bio, pBuffer,
-				    (size > encodingContext->nextOffset ? encodingContext->nextOffset : size));
+				status = BIO_read(bio, pBuffer, (int)rd);
 				if (status <= 0)
 					return (effectiveDataLen > 0 ? effectiveDataLen : status);
 
-				encodingContext->nextOffset -= status;
+				encodingContext->nextOffset -= WINPR_ASSERTING_INT_CAST(uint32_t, status);
 				if (encodingContext->nextOffset == 0)
 				{
 					encodingContext->state = ChunkStateFooter;
@@ -1149,7 +1155,7 @@ int http_chuncked_read(BIO* bio, BYTE* pBuffer, size_t size,
 					return effectiveDataLen;
 
 				pBuffer += status;
-				size -= status;
+				size -= (size_t)status;
 			}
 			break;
 			case ChunkStateFooter:
@@ -1158,10 +1164,10 @@ int http_chuncked_read(BIO* bio, BYTE* pBuffer, size_t size,
 				WINPR_ASSERT(encodingContext->nextOffset == 0);
 				WINPR_ASSERT(encodingContext->headerFooterPos < 2);
 				ERR_clear_error();
-				status = BIO_read(bio, _dummy, 2 - encodingContext->headerFooterPos);
+				status = BIO_read(bio, _dummy, (int)(2 - encodingContext->headerFooterPos));
 				if (status >= 0)
 				{
-					encodingContext->headerFooterPos += status;
+					encodingContext->headerFooterPos += (size_t)status;
 					if (encodingContext->headerFooterPos == 2)
 					{
 						encodingContext->state = ChunkStateLenghHeader;
@@ -1185,7 +1191,7 @@ int http_chuncked_read(BIO* bio, BYTE* pBuffer, size_t size,
 					{
 						if (*dst == '\n')
 							_haveNewLine = TRUE;
-						encodingContext->headerFooterPos += status;
+						encodingContext->headerFooterPos += (size_t)status;
 						dst += status;
 					}
 					else
@@ -1302,13 +1308,13 @@ static SSIZE_T http_response_recv_line(rdpTls* tls, HttpResponse* response)
 			goto out_error;
 		}
 
-		/* Always check at most the lase 8 bytes for occurance of the desired
+		/* Always check at most the lase 8 bytes for occurrence of the desired
 		 * sequence of \r\n\r\n */
 		s = (position > 8) ? 8 : position;
 		end = (char*)Stream_Pointer(response->data) - s;
 
 		if (string_strnstr(end, "\r\n\r\n", s) != NULL)
-			payloadOffset = Stream_GetPosition(response->data);
+			payloadOffset = WINPR_ASSERTING_INT_CAST(SSIZE_T, Stream_GetPosition(response->data));
 	}
 
 out_error:
@@ -1352,7 +1358,7 @@ static BOOL http_response_recv_body(rdpTls* tls, HttpResponse* response, BOOL re
 				full_len += status;
 			}
 		} while (ctx.state != ChunkStateEnd);
-		response->BodyLength = full_len;
+		response->BodyLength = WINPR_ASSERTING_INT_CAST(uint32_t, full_len);
 		if (response->BodyLength > 0)
 			response->BodyContent = &(Stream_Buffer(response->data))[payloadOffset];
 	}
@@ -1366,8 +1372,10 @@ static BOOL http_response_recv_body(rdpTls* tls, HttpResponse* response, BOOL re
 				goto out_error;
 
 			ERR_clear_error();
-			status = BIO_read(tls->bio, Stream_Pointer(response->data),
-			                  bodyLength - response->BodyLength);
+			size_t diff = bodyLength - response->BodyLength;
+			if (diff > INT32_MAX)
+				diff = INT32_MAX;
+			status = BIO_read(tls->bio, Stream_Pointer(response->data), (int)diff);
 
 			if (status <= 0)
 			{
@@ -1427,11 +1435,13 @@ HttpResponse* http_response_recv(rdpTls* tls, BOOL readContentLength)
 	if (payloadOffset)
 	{
 		size_t count = 0;
-		char* buffer = (char*)Stream_Buffer(response->data);
-		char* line = (char*)Stream_Buffer(response->data);
+		char* buffer = Stream_BufferAs(response->data, char);
+		char* line = Stream_BufferAs(response->data, char);
 		char* context = NULL;
 
-		while ((line = string_strnstr(line, "\r\n", payloadOffset - (line - buffer) - 2UL)))
+		while ((line = string_strnstr(line, "\r\n",
+		                              WINPR_ASSERTING_INT_CAST(size_t, payloadOffset) -
+		                                  WINPR_ASSERTING_INT_CAST(size_t, (line - buffer)) - 2UL)))
 		{
 			line += 2;
 			count++;
@@ -1462,7 +1472,8 @@ HttpResponse* http_response_recv(rdpTls* tls, BOOL readContentLength)
 		if (!http_response_parse_header(response))
 			goto out_error;
 
-		response->BodyLength = Stream_GetPosition(response->data) - payloadOffset;
+		response->BodyLength =
+		    Stream_GetPosition(response->data) - WINPR_ASSERTING_INT_CAST(size_t, payloadOffset);
 
 		WINPR_ASSERT(response->BodyLength == 0);
 		bodyLength = response->BodyLength; /* expected body length */
@@ -1495,7 +1506,8 @@ HttpResponse* http_response_recv(rdpTls* tls, BOOL readContentLength)
 		}
 
 		/* Fetch remaining body! */
-		if (!http_response_recv_body(tls, response, readContentLength, payloadOffset, bodyLength))
+		if (!http_response_recv_body(tls, response, readContentLength,
+		                             WINPR_ASSERTING_INT_CAST(size_t, payloadOffset), bodyLength))
 			goto out_error;
 	}
 	Stream_SealLength(response->data);
@@ -1574,7 +1586,7 @@ void http_response_free(HttpResponse* response)
 	if (!response)
 		return;
 
-	free(response->lines);
+	free((void*)response->lines);
 	ListDictionary_Free(response->Authenticates);
 	ListDictionary_Free(response->SetCookie);
 	Stream_Free(response->data, TRUE);
@@ -1606,7 +1618,7 @@ BOOL http_request_set_content_length(HttpRequest* request, size_t length)
 	return TRUE;
 }
 
-long http_response_get_status_code(const HttpResponse* response)
+INT16 http_response_get_status_code(const HttpResponse* response)
 {
 	WINPR_ASSERT(response);
 
@@ -1617,7 +1629,7 @@ size_t http_response_get_body_length(const HttpResponse* response)
 {
 	WINPR_ASSERT(response);
 
-	return (SSIZE_T)response->BodyLength;
+	return response->BodyLength;
 }
 
 const char* http_response_get_auth_token(const HttpResponse* response, const char* method)
