@@ -728,7 +728,7 @@ struct rdp_rpc
 	UINT32 result;
 
 	rdpCredsspAuth* auth;
-	size_t SendSeqNum;
+	UINT32 SendSeqNum;
 
 	RpcClient* client;
 
@@ -773,8 +773,9 @@ FREERDP_LOCAL SSIZE_T rpc_channel_read(RpcChannel* channel, wStream* s, size_t l
 
 FREERDP_LOCAL void rpc_channel_free(RpcChannel* channel);
 
+WINPR_ATTR_MALLOC(rpc_channel_free, 1)
 FREERDP_LOCAL RpcOutChannel* rpc_out_channel_new(rdpRpc* rpc, const GUID* guid);
-FREERDP_LOCAL int rpc_out_channel_replacement_connect(RpcOutChannel* outChannel, int timeout);
+FREERDP_LOCAL int rpc_out_channel_replacement_connect(RpcOutChannel* outChannel, uint32_t timeout);
 
 FREERDP_LOCAL BOOL rpc_in_channel_transition_to_state(RpcInChannel* inChannel,
                                                       CLIENT_IN_CHANNEL_STATE state);
@@ -787,7 +788,9 @@ FREERDP_LOCAL BOOL rpc_virtual_connection_transition_to_state(rdpRpc* rpc,
 
 FREERDP_LOCAL BOOL rpc_connect(rdpRpc* rpc, UINT32 timeout);
 
-FREERDP_LOCAL rdpRpc* rpc_new(rdpTransport* transport);
 FREERDP_LOCAL void rpc_free(rdpRpc* rpc);
 
+WINPR_ATTR_MALLOC(rpc_free, 1)
+FREERDP_LOCAL rdpRpc* rpc_new(rdpTransport* transport);
+
 #endif /* FREERDP_LIB_CORE_GATEWAY_RPC_H */
