@@ -24,6 +24,7 @@
 #include <winpr/crt.h>
 #include <winpr/wtypes.h>
 #include <winpr/assert.h>
+#include <winpr/cast.h>
 #include <winpr/print.h>
 #include <winpr/synch.h>
 #include <winpr/thread.h>
@@ -45,6 +46,7 @@
 
 static const char* rpc_client_state_str(RPC_CLIENT_STATE state)
 {
+	// NOLINTNEXTLINE(clang-analyzer-deadcode.DeadStores)
 	const char* str = "RPC_CLIENT_STATE_UNKNOWN";
 
 	switch (state)
@@ -80,6 +82,8 @@ static const char* rpc_client_state_str(RPC_CLIENT_STATE state)
 		case RPC_CLIENT_STATE_FINAL:
 			str = "RPC_CLIENT_STATE_FINAL";
 			break;
+		default:
+			break;
 	}
 	return str;
 }
@@ -135,7 +139,7 @@ static int rpc_client_receive_pipe_write(RpcClient* client, const BYTE* buffer,
 		status += (int)length;
 
 	if (ringbuffer_used(&(client->ReceivePipe)) > 0)
-		SetEvent(client->PipeEvent);
+		(void)SetEvent(client->PipeEvent);
 
 	LeaveCriticalSection(&(client->PipeLock));
 	return status;
@@ -163,7 +167,7 @@ int rpc_client_receive_pipe_read(RpcClient* client, BYTE* buffer, size_t length)
 		ringbuffer_commit_read_bytes(&(client->ReceivePipe), status);
 
 	if (ringbuffer_used(&(client->ReceivePipe)) < 1)
-		ResetEvent(client->PipeEvent);
+		(void)ResetEvent(client->PipeEvent);
 
 	LeaveCriticalSection(&(client->PipeLock));
 
@@ -263,6 +267,8 @@ static int rpc_client_recv_pdu_int(rdpRpc* rpc, RPC_PDU* pdu)
 
 			case VIRTUAL_CONNECTION_STATE_FINAL:
 				break;
+			default:
+				break;
 		}
 	}
 	else if (rpc->State < RPC_CLIENT_STATE_CONTEXT_NEGOTIATED)
@@ -311,6 +317,9 @@ static int rpc_client_recv_pdu_int(rdpRpc* rpc, RPC_PDU* pdu)
 						WLog_ERR(TAG, "tsg_proxy_begin failure");
 						return -1;
 					}
+					break;
+				default:
+					break;
 			}
 
 			status = 1;
@@ -559,7 +568,6 @@ fail:
 static SSIZE_T rpc_client_default_out_channel_recv(rdpRpc* rpc)
 {
 	SSIZE_T status = -1;
-	UINT32 statusCode = 0;
 	HttpResponse* response = NULL;
 	RpcInChannel* inChannel = NULL;
 	RpcOutChannel* outChannel = NULL;
@@ -638,7 +646,7 @@ static SSIZE_T rpc_client_default_out_channel_recv(rdpRpc* rpc)
 		if (!response)
 			return -1;
 
-		statusCode = http_response_get_status_code(response);
+		const INT16 statusCode = http_response_get_status_code(response);
 
 		if (statusCode != HTTP_STATUS_OK)
 		{
@@ -1004,14 +1012,16 @@ int rpc_in_channel_send_pdu(RpcInChannel* inChannel, const BYTE* buffer, size_t
 		inChannel->SenderAvailableWindow -= status;
 	}
 
-	return status;
+	if (status > INT32_MAX)
+		return -1;
+	return (int)status;
 }
 
 BOOL rpc_client_write_call(rdpRpc* rpc, wStream* s, UINT16 opnum)
 {
 	size_t offset = 0;
 	BYTE* buffer = NULL;
-	UINT32 stub_data_pad = 0;
+	size_t stub_data_pad = 0;
 	SecBuffer plaintext;
 	SecBuffer ciphertext = { 0 };
 	RpcClientCall* clientCall = NULL;
@@ -1019,8 +1029,6 @@ BOOL rpc_client_write_call(rdpRpc* rpc, wStream* s, UINT16 opnum)
 	rpcconn_request_hdr_t request_pdu = { 0 };
 	RpcVirtualConnection* connection = NULL;
 	RpcInChannel* inChannel = NULL;
-	size_t length = 0;
-	size_t size = 0;
 	BOOL rc = FALSE;
 
 	if (!s)
@@ -1047,16 +1055,18 @@ BOOL rpc_client_write_call(rdpRpc* rpc, wStream* s, UINT16 opnum)
 		goto fail;
 
 	Stream_SealLength(s);
-	length = Stream_Length(s);
+	const size_t length = Stream_Length(s);
+	if (length > UINT32_MAX)
+		goto fail;
 
-	size = credssp_auth_trailer_size(auth);
+	const size_t asize = credssp_auth_trailer_size(auth);
 
 	request_pdu.header = rpc_pdu_header_init(rpc);
 	request_pdu.header.ptype = PTYPE_REQUEST;
 	request_pdu.header.pfc_flags = PFC_FIRST_FRAG | PFC_LAST_FRAG;
-	request_pdu.header.auth_length = (UINT16)size;
+	request_pdu.header.auth_length = (UINT16)asize;
 	request_pdu.header.call_id = rpc->CallId++;
-	request_pdu.alloc_hint = length;
+	request_pdu.alloc_hint = (UINT32)length;
 	request_pdu.p_cont_id = 0x0000;
 	request_pdu.opnum = opnum;
 	clientCall = rpc_client_call_new(request_pdu.header.call_id, request_pdu.opnum);
@@ -1078,14 +1088,20 @@ BOOL rpc_client_write_call(rdpRpc* rpc, wStream* s, UINT16 opnum)
 	offset = 24;
 	stub_data_pad = rpc_offset_align(&offset, 8);
 	offset += length;
-	request_pdu.auth_verifier.auth_pad_length = rpc_offset_align(&offset, 4);
+
+	const size_t alg = rpc_offset_align(&offset, 4);
+	WINPR_ASSERT(alg <= UINT8_MAX);
+	request_pdu.auth_verifier.auth_pad_length = (UINT8)alg;
 	request_pdu.auth_verifier.auth_type =
 	    rpc_auth_pkg_to_security_provider(credssp_auth_pkg_name(rpc->auth));
 	request_pdu.auth_verifier.auth_level = RPC_C_AUTHN_LEVEL_PKT_INTEGRITY;
 	request_pdu.auth_verifier.auth_reserved = 0x00;
 	request_pdu.auth_verifier.auth_context_id = 0x00000000;
 	offset += (8 + request_pdu.header.auth_length);
-	request_pdu.header.frag_length = offset;
+
+	if (offset > UINT16_MAX)
+		goto fail;
+	request_pdu.header.frag_length = (UINT16)offset;
 	buffer = (BYTE*)calloc(1, request_pdu.header.frag_length);
 
 	if (!buffer)
@@ -1100,9 +1116,14 @@ BOOL rpc_client_write_call(rdpRpc* rpc, wStream* s, UINT16 opnum)
 	CopyMemory(&buffer[offset], &request_pdu.auth_verifier.auth_type, 8);
 	offset += 8;
 
+	if (offset > UINT32_MAX)
+		goto fail;
+
 	plaintext.pvBuffer = buffer;
-	plaintext.cbBuffer = offset;
+	plaintext.cbBuffer = (UINT32)offset;
 	plaintext.BufferType = SECBUFFER_READONLY;
+
+	size_t size = 0;
 	if (!credssp_auth_encrypt(auth, &plaintext, &ciphertext, &size, rpc->SendSeqNum++))
 		goto fail;
 
@@ -1212,7 +1233,7 @@ void rpc_client_free(RpcClient* client)
 		Stream_Free(client->ReceiveFragment, TRUE);
 
 	if (client->PipeEvent)
-		CloseHandle(client->PipeEvent);
+		(void)CloseHandle(client->PipeEvent);
 
 	ringbuffer_destroy(&(client->ReceivePipe));
 	DeleteCriticalSection(&(client->PipeLock));
