@@ -45,10 +45,10 @@
 
 #define TAG FREERDP_TAG("core.license")
 
-#if 0
-#define LICENSE_NULL_CLIENT_RANDOM 1
-#define LICENSE_NULL_PREMASTER_SECRET 1
-#endif
+// #define LICENSE_NULL_CLIENT_RANDOM 1
+// #define LICENSE_NULL_PREMASTER_SECRET 1
+
+// #define WITH_LICENSE_DECRYPT_CHALLENGE_RESPONSE
 
 #define PLATFORM_CHALLENGE_RESPONSE_VERSION 0x0100
 
@@ -65,12 +65,13 @@ enum LicenseRequestType
 	ERROR_ALERT = 0xFF
 };
 
+/*
 #define LICENSE_PKT_CS_MASK \
-	(LICENSE_INFO | NEW_LICENSE_REQUEST | PLATFORM_CHALLENGE_RESPONSE | ERROR_ALERT)
+    (LICENSE_INFO | NEW_LICENSE_REQUEST | PLATFORM_CHALLENGE_RESPONSE | ERROR_ALERT)
 #define LICENSE_PKT_SC_MASK \
-	(LICENSE_REQUEST | PLATFORM_CHALLENGE | NEW_LICENSE | UPGRADE_LICENSE | ERROR_ALERT)
+    (LICENSE_REQUEST | PLATFORM_CHALLENGE | NEW_LICENSE | UPGRADE_LICENSE | ERROR_ALERT)
 #define LICENSE_PKT_MASK (LICENSE_PKT_CS_MASK | LICENSE_PKT_SC_MASK)
-
+*/
 #define LICENSE_PREAMBLE_LENGTH 4
 
 /* Cryptographic Lengths */
@@ -82,15 +83,15 @@ enum LicenseRequestType
 #define MAC_SALT_KEY_LENGTH 16
 #define LICENSING_ENCRYPTION_KEY_LENGTH 16
 #define HWID_PLATFORM_ID_LENGTH 4
-#define HWID_UNIQUE_DATA_LENGTH 16
+// #define HWID_UNIQUE_DATA_LENGTH 16
 #define HWID_LENGTH 20
-#define LICENSING_PADDING_SIZE 8
+// #define LICENSING_PADDING_SIZE 8
 
 /* Preamble Flags */
 
-#define PREAMBLE_VERSION_2_0 0x02
+// #define PREAMBLE_VERSION_2_0 0x02
 #define PREAMBLE_VERSION_3_0 0x03
-#define LicenseProtocolVersionMask 0x0F
+// #define LicenseProtocolVersionMask 0x0F
 #define EXTENDED_ERROR_MSG_SUPPORTED 0x80
 
 /** @brief binary Blob Types */
@@ -202,6 +203,7 @@ struct rdp_license
 	LICENSE_BLOB* ClientUserName;
 	LICENSE_BLOB* ClientMachineName;
 	LICENSE_BLOB* PlatformChallenge;
+	LICENSE_BLOB* PlatformChallengeResponse;
 	LICENSE_BLOB* EncryptedPremasterSecret;
 	LICENSE_BLOB* EncryptedPlatformChallenge;
 	LICENSE_BLOB* EncryptedPlatformChallengeResponse;
@@ -236,7 +238,7 @@ static const char* license_preferred_key_exchange_alg_string(UINT32 alg, char* b
 			break;
 	}
 
-	_snprintf(buffer, size, "%s [0x%08" PRIx32 "]", name, alg);
+	(void)_snprintf(buffer, size, "%s [0x%08" PRIx32 "]", name, alg);
 	return buffer;
 }
 
@@ -330,12 +332,11 @@ static BOOL license_read_new_license_request_packet(rdpLicense* license, wStream
 static BOOL license_answer_license_request(rdpLicense* license);
 
 static BOOL license_send_platform_challenge_response(rdpLicense* license);
-static BOOL license_read_platform_challenge_response(rdpLicense* license, wStream* s);
+static BOOL license_read_platform_challenge_response(rdpLicense* license);
 
 static BOOL license_read_client_platform_challenge_response(rdpLicense* license, wStream* s);
 static BOOL license_write_client_platform_challenge_response(rdpLicense* license, wStream* s);
 
-static BOOL license_read_server_upgrade_license(rdpLicense* license, wStream* s);
 static BOOL license_write_server_upgrade_license(const rdpLicense* license, wStream* s);
 
 static BOOL license_send_license_info(rdpLicense* license, const LICENSE_BLOB* calBlob,
@@ -348,23 +349,22 @@ static state_run_t license_server_recv(rdpLicense* license, wStream* s);
 
 #ifdef WITH_DEBUG_LICENSE
 
-static const char* const error_codes[] = { "ERR_UNKNOWN",
-	                                       "ERR_INVALID_SERVER_CERTIFICATE",
-	                                       "ERR_NO_LICENSE",
-	                                       "ERR_INVALID_MAC",
-	                                       "ERR_INVALID_SCOPE",
-	                                       "ERR_UNKNOWN",
-	                                       "ERR_NO_LICENSE_SERVER",
-	                                       "STATUS_VALID_CLIENT",
-	                                       "ERR_INVALID_CLIENT",
-	                                       "ERR_UNKNOWN",
-	                                       "ERR_UNKNOWN",
-	                                       "ERR_INVALID_PRODUCT_ID",
-	                                       "ERR_INVALID_MESSAGE_LENGTH" };
+static const char* error_codes[] = { "ERR_UNKNOWN",
+	                                 "ERR_INVALID_SERVER_CERTIFICATE",
+	                                 "ERR_NO_LICENSE",
+	                                 "ERR_INVALID_MAC",
+	                                 "ERR_INVALID_SCOPE",
+	                                 "ERR_UNKNOWN",
+	                                 "ERR_NO_LICENSE_SERVER",
+	                                 "STATUS_VALID_CLIENT",
+	                                 "ERR_INVALID_CLIENT",
+	                                 "ERR_UNKNOWN",
+	                                 "ERR_UNKNOWN",
+	                                 "ERR_INVALID_PRODUCT_ID",
+	                                 "ERR_INVALID_MESSAGE_LENGTH" };
 
-static const char* const state_transitions[] = { "ST_UNKNOWN", "ST_TOTAL_ABORT", "ST_NO_TRANSITION",
-	                                             "ST_RESET_PHASE_TO_START",
-	                                             "ST_RESEND_LAST_MESSAGE" };
+static const char* state_transitions[] = { "ST_UNKNOWN", "ST_TOTAL_ABORT", "ST_NO_TRANSITION",
+	                                       "ST_RESET_PHASE_TO_START", "ST_RESEND_LAST_MESSAGE" };
 
 static void license_print_product_info(const LICENSE_PRODUCT_INFO* productInfo)
 {
@@ -491,7 +491,7 @@ static BOOL computeCalHash(const char* hostname, char* hashStr, size_t len)
 		goto out;
 
 	for (size_t i = 0; i < sizeof(hash); i++, hashStr += 2)
-		sprintf_s(hashStr, 3, "%.2x", hash[i]);
+		(void)sprintf_s(hashStr, 3, "%.2x", hash[i]);
 
 	ret = TRUE;
 out:
@@ -548,8 +548,8 @@ static BOOL saveCal(const rdpSettings* settings, const BYTE* data, size_t length
 
 	if (!computeCalHash(hostname, hash, sizeof(hash)))
 		goto out;
-	sprintf_s(filename, sizeof(filename) - 1, "%s.cal", hash);
-	sprintf_s(filenameNew, sizeof(filenameNew) - 1, "%s.cal.new", hash);
+	(void)sprintf_s(filename, sizeof(filename) - 1, "%s.cal", hash);
+	(void)sprintf_s(filenameNew, sizeof(filenameNew) - 1, "%s.cal.new", hash);
 
 	if (!(filepath = GetCombinedPath(licenseStorePath, filename)))
 	{
@@ -571,7 +571,7 @@ static BOOL saveCal(const rdpSettings* settings, const BYTE* data, size_t length
 	}
 
 	written = fwrite(data, length, 1, fp);
-	fclose(fp);
+	(void)fclose(fp);
 
 	if (written != 1)
 	{
@@ -612,7 +612,7 @@ static BYTE* loadCalFile(const rdpSettings* settings, const char* hostname, size
 		return NULL;
 	}
 
-	sprintf_s(calFilename, sizeof(calFilename) - 1, "%s.cal", hash);
+	(void)sprintf_s(calFilename, sizeof(calFilename) - 1, "%s.cal", hash);
 
 	if (!(licenseStorePath = GetCombinedPath(
 	          freerdp_settings_get_string(settings, FreeRDP_ConfigPath), licenseStore)))
@@ -625,9 +625,11 @@ static BYTE* loadCalFile(const rdpSettings* settings, const char* hostname, size
 	if (!fp)
 		goto error_open;
 
-	_fseeki64(fp, 0, SEEK_END);
+	if (_fseeki64(fp, 0, SEEK_END) != 0)
+		goto error_malloc;
 	length = _ftelli64(fp);
-	_fseeki64(fp, 0, SEEK_SET);
+	if (_fseeki64(fp, 0, SEEK_SET) != 0)
+		goto error_malloc;
 	if (length < 0)
 		goto error_malloc;
 
@@ -641,7 +643,7 @@ static BYTE* loadCalFile(const rdpSettings* settings, const char* hostname, size
 
 	*dataLen = (size_t)length;
 
-	fclose(fp);
+	(void)fclose(fp);
 	free(calPath);
 	free(licenseStorePath);
 	return ret;
@@ -727,7 +729,7 @@ wStream* license_send_stream_init(rdpLicense* license)
 	 * layer is used. If the peer has not indicated that it is capable of
 	 * processing encrypted licensing packets (rdp->do_crypt_license) we turn
 	 * off encryption (via rdp->do_crypt) before initializing the rdp stream
-	 * and reenable it afterwards.
+	 * and re-enable it afterwards.
 	 */
 
 	if (do_crypt)
@@ -784,7 +786,10 @@ static BOOL license_send(rdpLicense* license, wStream* s, BYTE type)
 		flags |= EXTENDED_ERROR_MSG_SUPPORTED;
 
 	if (!license_write_preamble(s, type, flags, wMsgSize))
+	{
+		Stream_Release(s);
 		return FALSE;
+	}
 
 #ifdef WITH_DEBUG_LICENSE
 	WLog_DBG(TAG, "Sending %s Packet, length %" PRIu16 "", license_request_type_string(type),
@@ -797,19 +802,6 @@ static BOOL license_send(rdpLicense* license, wStream* s, BYTE type)
 	return ret;
 }
 
-BOOL license_read_server_upgrade_license(rdpLicense* license, wStream* s)
-{
-	WINPR_ASSERT(license);
-
-	if (!license_read_binary_blob(s, license->EncryptedLicenseInfo))
-		return FALSE;
-	if (!license_check_stream_length(s, sizeof(license->MACData),
-	                                 "SERVER_UPGRADE_LICENSE::MACData"))
-		return FALSE;
-	Stream_Read(s, license->MACData, sizeof(license->MACData));
-	return TRUE;
-}
-
 BOOL license_write_server_upgrade_license(const rdpLicense* license, wStream* s)
 {
 	WINPR_ASSERT(license);
@@ -1114,10 +1106,9 @@ BOOL license_generate_hwid(rdpLicense* license)
 		const char* hostname = license->rdp->settings->ClientHostname;
 		wStream* s = Stream_StaticInit(&buffer, license->HardwareId, 4);
 		Stream_Write_UINT32(s, license->PlatformId);
-		Stream_Free(s, TRUE);
 
 		hashTarget = (const BYTE*)hostname;
-		targetLen = strlen(hostname);
+		targetLen = hostname ? strlen(hostname) : 0;
 	}
 
 	/* Allow FIPS override for use of MD5 here, really this does not have to be MD5 as we are just
@@ -1220,7 +1211,9 @@ static BOOL license_rc4_with_licenseKey(const rdpLicense* license, const BYTE* i
 		return FALSE;
 	}
 
-	BYTE* buffer = (BYTE*)realloc(target->data, len);
+	BYTE* buffer = NULL;
+	if (len > 0)
+		buffer = realloc(target->data, len);
 	if (!buffer)
 		goto error_buffer;
 
@@ -1469,7 +1462,9 @@ BOOL license_read_binary_blob_data(LICENSE_BLOB* blob, UINT16 wBlobType, const v
 	}
 
 	blob->type = wBlobType;
-	blob->data = (BYTE*)malloc(blob->length);
+	blob->data = NULL;
+	if (blob->length > 0)
+		blob->data = malloc(blob->length);
 	if (!blob->data)
 	{
 		WLog_ERR(TAG, "license binary blob::length=%" PRIu16 ", blob::data=%p", blob->length,
@@ -1614,7 +1609,7 @@ BOOL license_read_scope_list(wStream* s, SCOPE_LIST* scopeList)
 
 	Stream_Read_UINT32(s, scopeCount); /* ScopeCount (4 bytes) */
 
-	if (!license_check_stream_length(s, scopeCount * 4ull, "license scope list::count"))
+	if (!license_check_stream_length(s, 4ll * scopeCount, "license scope list::count"))
 		return FALSE;
 
 	if (!license_scope_list_resize(scopeList, scopeCount))
@@ -1679,14 +1674,15 @@ BOOL license_scope_list_resize(SCOPE_LIST* scopeList, UINT32 count)
 
 	if (count > 0)
 	{
-		LICENSE_BLOB** tmp = realloc(scopeList->array, count * sizeof(LICENSE_BLOB*));
+		LICENSE_BLOB** tmp =
+		    (LICENSE_BLOB**)realloc((void*)scopeList->array, count * sizeof(LICENSE_BLOB*));
 		if (!tmp)
 			return FALSE;
 		scopeList->array = tmp;
 	}
 	else
 	{
-		free(scopeList->array);
+		free((void*)scopeList->array);
 		scopeList->array = NULL;
 	}
 
@@ -1723,20 +1719,21 @@ void license_free_scope_list(SCOPE_LIST* scopeList)
 BOOL license_send_license_info(rdpLicense* license, const LICENSE_BLOB* calBlob,
                                const BYTE* signature, size_t signature_length)
 {
-	wStream* s = license_send_stream_init(license);
-
 	WINPR_ASSERT(calBlob);
 	WINPR_ASSERT(signature);
 	WINPR_ASSERT(license->certificate);
 
 	const rdpCertInfo* info = freerdp_certificate_get_info(license->certificate);
+	if (!info)
+		return FALSE;
 
+	wStream* s = license_send_stream_init(license);
 	if (!s)
 		return FALSE;
 
 	if (!license_check_stream_capacity(s, 8 + sizeof(license->ClientRandom),
 	                                   "license info::ClientRandom"))
-		return FALSE;
+		goto error;
 
 	Stream_Write_UINT32(s,
 	                    license->PreferredKeyExchangeAlg); /* PreferredKeyExchangeAlg (4 bytes) */
@@ -1953,7 +1950,6 @@ fail:
 BOOL license_read_platform_challenge_packet(rdpLicense* license, wStream* s)
 {
 	BYTE macData[LICENSING_ENCRYPTION_KEY_LENGTH] = { 0 };
-	UINT32 ConnectFlags = 0;
 
 	WINPR_ASSERT(license);
 
@@ -1962,7 +1958,9 @@ BOOL license_read_platform_challenge_packet(rdpLicense* license, wStream* s)
 	if (!license_check_stream_length(s, 4, "license platform challenge"))
 		return FALSE;
 
-	Stream_Read_UINT32(s, ConnectFlags); /* ConnectFlags, Reserved (4 bytes) */
+	/* [MS-RDPELE] 2.2.2.4 Server Platform Challenge (SERVER_PLATFORM_CHALLENGE)
+	 * reserved field */
+	Stream_Seek_UINT32(s); /* ConnectFlags, Reserved (4 bytes) */
 
 	/* EncryptedPlatformChallenge */
 	license->EncryptedPlatformChallenge->type = BB_ANY_BLOB;
@@ -2473,7 +2471,6 @@ BOOL license_answer_license_request(rdpLicense* license)
 
 BOOL license_send_platform_challenge_response(rdpLicense* license)
 {
-	wStream* s = license_send_stream_init(license);
 	wStream* challengeRespData = NULL;
 	BYTE* buffer = NULL;
 	BOOL status = 0;
@@ -2546,6 +2543,10 @@ BOOL license_send_platform_challenge_response(rdpLicense* license)
 	winpr_HexDump(TAG, WLOG_DEBUG, license->EncryptedHardwareId->data,
 	              license->EncryptedHardwareId->length);
 #endif
+	wStream* s = license_send_stream_init(license);
+	if (!s)
+		return FALSE;
+
 	if (license_write_client_platform_challenge_response(license, s))
 		return license_send(license, s, PLATFORM_CHALLENGE_RESPONSE);
 
@@ -2553,12 +2554,8 @@ BOOL license_send_platform_challenge_response(rdpLicense* license)
 	return FALSE;
 }
 
-BOOL license_read_platform_challenge_response(rdpLicense* license, wStream* s)
+BOOL license_read_platform_challenge_response(WINPR_ATTR_UNUSED rdpLicense* license)
 {
-	UINT16 wVersion = 0;
-	UINT16 cbChallenge = 0;
-	const BYTE* pbChallenge = NULL;
-
 	WINPR_ASSERT(license);
 	WINPR_ASSERT(license->PlatformChallenge);
 	WINPR_ASSERT(license->MacSaltKey);
@@ -2567,9 +2564,25 @@ BOOL license_read_platform_challenge_response(rdpLicense* license, wStream* s)
 
 	DEBUG_LICENSE("Receiving Platform Challenge Response Packet");
 
-	if (!license_check_stream_length(s, 8, "PLATFORM_CHALLENGE_RESPONSE_DATA"))
+#if defined(WITH_LICENSE_DECRYPT_CHALLENGE_RESPONSE)
+	BOOL rc = FALSE;
+	LICENSE_BLOB* dblob = license_new_binary_blob(BB_ANY_BLOB);
+	if (!dblob)
 		return FALSE;
 
+	wStream sbuffer = { 0 };
+	UINT16 wVersion = 0;
+	UINT16 cbChallenge = 0;
+	const BYTE* pbChallenge = NULL;
+	LICENSE_BLOB* blob = license->EncryptedPlatformChallengeResponse;
+
+	if (!license_rc4_with_licenseKey(license, blob->data, blob->length, dblob))
+		goto fail;
+
+	wStream* s = Stream_StaticConstInit(&sbuffer, dblob->data, dblob->length);
+	if (!license_check_stream_length(s, 8, "PLATFORM_CHALLENGE_RESPONSE_DATA"))
+		goto fail;
+
 	Stream_Read_UINT16(s, wVersion);
 	if (wVersion != PLATFORM_CHALLENGE_RESPONSE_VERSION)
 	{
@@ -2577,7 +2590,7 @@ BOOL license_read_platform_challenge_response(rdpLicense* license, wStream* s)
 		          "Invalid PLATFORM_CHALLENGE_RESPONSE_DATA::wVersion 0x%04" PRIx16
 		          ", expected 0x04" PRIx16,
 		          wVersion, PLATFORM_CHALLENGE_RESPONSE_VERSION);
-		return FALSE;
+		goto fail;
 	}
 	Stream_Read_UINT16(s, license->ClientType);
 	Stream_Read_UINT16(s, license->LicenseDetailLevel);
@@ -2585,13 +2598,22 @@ BOOL license_read_platform_challenge_response(rdpLicense* license, wStream* s)
 
 	if (!license_check_stream_length(s, cbChallenge,
 	                                 "PLATFORM_CHALLENGE_RESPONSE_DATA::pbChallenge"))
-		return FALSE;
+		goto fail;
 
 	pbChallenge = Stream_Pointer(s);
-	if (!license_read_binary_blob_data(license->EncryptedPlatformChallengeResponse, BB_DATA_BLOB,
+	if (!license_read_binary_blob_data(license->PlatformChallengeResponse, BB_DATA_BLOB,
 	                                   pbChallenge, cbChallenge))
-		return FALSE;
-	return Stream_SafeSeek(s, cbChallenge);
+		goto fail;
+	if (!Stream_SafeSeek(s, cbChallenge))
+		goto fail;
+
+	rc = TRUE;
+fail:
+	license_free_binary_blob(dblob);
+	return rc;
+#else
+	return TRUE;
+#endif
 }
 
 BOOL license_write_client_platform_challenge_response(rdpLicense* license, wStream* s)
@@ -2621,7 +2643,7 @@ BOOL license_read_client_platform_challenge_response(rdpLicense* license, wStrea
 	                                 "CLIENT_PLATFORM_CHALLENGE_RESPONSE::MACData"))
 		return FALSE;
 	Stream_Read(s, license->MACData, sizeof(license->MACData));
-	return TRUE;
+	return license_read_platform_challenge_response(license);
 }
 
 /**
@@ -2685,6 +2707,8 @@ rdpLicense* license_new(rdpRdp* rdp)
 		goto out_error;
 	if (!(license->PlatformChallenge = license_new_binary_blob(BB_ANY_BLOB)))
 		goto out_error;
+	if (!(license->PlatformChallengeResponse = license_new_binary_blob(BB_ANY_BLOB)))
+		goto out_error;
 	if (!(license->EncryptedPlatformChallenge = license_new_binary_blob(BB_ANY_BLOB)))
 		goto out_error;
 	if (!(license->EncryptedPlatformChallengeResponse =
@@ -2729,6 +2753,7 @@ void license_free(rdpLicense* license)
 		license_free_binary_blob(license->ClientUserName);
 		license_free_binary_blob(license->ClientMachineName);
 		license_free_binary_blob(license->PlatformChallenge);
+		license_free_binary_blob(license->PlatformChallengeResponse);
 		license_free_binary_blob(license->EncryptedPlatformChallenge);
 		license_free_binary_blob(license->EncryptedPlatformChallengeResponse);
 		license_free_binary_blob(license->EncryptedPremasterSecret);
@@ -2802,18 +2827,25 @@ BOOL license_server_send_request(rdpLicense* license)
 	return license_set_state(license, LICENSE_STATE_REQUEST);
 }
 
-static BOOL license_set_string(const char* what, const char* value, WCHAR** dst, UINT32* dstLen)
+static BOOL license_set_string(const char* what, const char* value, BYTE** bdst, UINT32* dstLen)
 {
 	WINPR_ASSERT(what);
 	WINPR_ASSERT(value);
-	WINPR_ASSERT(dst);
+	WINPR_ASSERT(bdst);
 	WINPR_ASSERT(dstLen);
 
-	size_t len = 0;
-	*dst = (BYTE*)ConvertUtf8ToWCharAlloc(value, &len);
-	if (!*dst || (len > UINT32_MAX / sizeof(WCHAR)))
+	union
 	{
-		WLog_ERR(TAG, "license->ProductInfo: %s == %p || %" PRIu32 " > UINT32_MAX", what, *dst,
+		WCHAR** w;
+		BYTE** b;
+	} cnv;
+	cnv.b = bdst;
+
+	size_t len = 0;
+	*cnv.w = ConvertUtf8ToWCharAlloc(value, &len);
+	if (!*cnv.w || (len > UINT32_MAX / sizeof(WCHAR)))
+	{
+		WLog_ERR(TAG, "license->ProductInfo: %s == %p || %" PRIu32 " > UINT32_MAX", what, *cnv.w,
 		         len);
 		return FALSE;
 	}
@@ -2840,8 +2872,9 @@ BOOL license_server_configure(rdpLicense* license)
 	    freerdp_settings_get_uint32(settings, FreeRDP_ServerLicenseProductVersion);
 	const UINT32 issuerCount =
 	    freerdp_settings_get_uint32(settings, FreeRDP_ServerLicenseProductIssuersCount);
-	const char* const* issuers = (const char* const*)freerdp_settings_get_pointer(
-	    settings, FreeRDP_ServerLicenseProductIssuers);
+
+	const void* ptr = freerdp_settings_get_pointer(settings, FreeRDP_ServerLicenseProductIssuers);
+	const char** issuers = WINPR_REINTERPRET_CAST(ptr, const void*, const char**);
 
 	WINPR_ASSERT(CompanyName);
 	WINPR_ASSERT(ProductName);
