@@ -94,25 +94,25 @@ static SSIZE_T crypto_rsa_common(const BYTE* input, size_t length, UINT32 key_le
 	if (!(y = BN_new()))
 		goto fail;
 
-	if (!BN_bin2bn(modulus_reverse, key_length, mod))
+	if (!BN_bin2bn(modulus_reverse, (int)key_length, mod))
 		goto fail;
 
-	if (!BN_bin2bn(exponent_reverse, exponent_size, exp))
+	if (!BN_bin2bn(exponent_reverse, (int)exponent_size, exp))
 		goto fail;
-	if (!BN_bin2bn(input_reverse, length, x))
+	if (!BN_bin2bn(input_reverse, (int)length, x))
 		goto fail;
 	if (BN_mod_exp(y, x, exp, mod, ctx) != 1)
 		goto fail;
 	output_length = BN_bn2bin(y, output);
 	if (output_length < 0)
 		goto fail;
-	if ((size_t)output_length > out_length)
+	if (WINPR_ASSERTING_INT_CAST(size_t, output_length) > out_length)
 		goto fail;
-	crypto_reverse(output, output_length);
+	crypto_reverse(output, WINPR_ASSERTING_INT_CAST(size_t, output_length));
 
 	if ((size_t)output_length < key_length)
 	{
-		size_t diff = key_length - output_length;
+		size_t diff = key_length - WINPR_ASSERTING_INT_CAST(size_t, output_length);
 		if ((size_t)output_length + diff > out_length)
 			diff = out_length - (size_t)output_length;
 		memset(output + output_length, 0, diff);
@@ -209,7 +209,7 @@ char* crypto_read_pem(const char* WINPR_RESTRICT filename, size_t* WINPR_RESTRIC
 	if (rc < 0)
 		goto fail;
 
-	pem = calloc(size + 1, sizeof(char));
+	pem = calloc(WINPR_ASSERTING_INT_CAST(size_t, size) + 1, sizeof(char));
 	if (!pem)
 		goto fail;
 
@@ -218,7 +218,7 @@ char* crypto_read_pem(const char* WINPR_RESTRICT filename, size_t* WINPR_RESTRIC
 		goto fail;
 
 	if (plength)
-		*plength = strnlen(pem, size);
+		*plength = strnlen(pem, WINPR_ASSERTING_INT_CAST(size_t, size));
 	(void)fclose(fp);
 	return pem;
 
