@@ -37,11 +37,11 @@
 
 #define VGIDS_EFID_MASTER 0xA000
 #define VGIDS_EFID_COMMON 0xA010
-#define VGIDS_EFID_CARDCF VGIDS_EFID_COMMON
-#define VGIDS_EFID_CARDAPPS VGIDS_EFID_COMMON
-#define VGIDS_EFID_CMAPFILE VGIDS_EFID_COMMON
+// #define VGIDS_EFID_CARDCF VGIDS_EFID_COMMON
+// #define VGIDS_EFID_CARDAPPS VGIDS_EFID_COMMON
+// #define VGIDS_EFID_CMAPFILE VGIDS_EFID_COMMON
 #define VGIDS_EFID_CARDID 0xA012
-#define VGIDS_EFID_KXC00 VGIDS_EFID_COMMON
+// #define VGIDS_EFID_KXC00 VGIDS_EFID_COMMON
 #define VGIDS_EFID_CURRENTDF 0x3FFF
 
 #define VGIDS_DO_FILESYSTEMTABLE 0xDF1F
@@ -58,34 +58,34 @@
 #define VGIDS_DEFAULT_RETRY_COUNTER 3
 
 #define VGIDS_KEY_TYPE_KEYEXCHANGE 0x9A
-#define VGIDS_KEY_TYPE_SIGNATURE 0x9C
+// #define VGIDS_KEY_TYPE_SIGNATURE 0x9C
 
 #define VGIDS_ALGID_RSA_1024 0x06
 #define VGIDS_ALGID_RSA_2048 0x07
 #define VGIDS_ALGID_RSA_3072 0x08
 #define VGIDS_ALGID_RSA_4096 0x09
 
-#define VGIDS_SE_CRT_AUTH 0xA4
+// #define VGIDS_SE_CRT_AUTH 0xA4
 #define VGIDS_SE_CRT_SIGN 0xB6
 #define VGIDS_SE_CRT_CONF 0xB8
 
 #define VGIDS_SE_ALGOID_CT_PAD_PKCS1 0x40
 #define VGIDS_SE_ALGOID_CT_PAD_OAEP 0x80
-#define VGIDS_SE_ALGOID_CT_RSA_1024 0x06
-#define VGIDS_SE_ALGOID_CT_RSA_2048 0x07
-#define VGIDS_SE_ALGOID_CT_RSA_3072 0x08
-#define VGIDS_SE_ALGOID_CT_RSA_4096 0x09
+// #define VGIDS_SE_ALGOID_CT_RSA_1024 0x06
+// #define VGIDS_SE_ALGOID_CT_RSA_2048 0x07
+// #define VGIDS_SE_ALGOID_CT_RSA_3072 0x08
+// #define VGIDS_SE_ALGOID_CT_RSA_4096 0x09
 
 #define VGIDS_SE_ALGOID_DST_PAD_PKCS1 0x40
-#define VGIDS_SE_ALGOID_DST_RSA_1024 0x06
-#define VGIDS_SE_ALGOID_DST_RSA_2048 0x07
-#define VGIDS_SE_ALGOID_DST_RSA_3072 0x08
-#define VGIDS_SE_ALGOID_DST_RSA_4096 0x09
-#define VGIDS_SE_ALGOID_DST_ECDSA_P192 0x0A
-#define VGIDS_SE_ALGOID_DST_ECDSA_P224 0x0B
-#define VGIDS_SE_ALGOID_DST_ECDSA_P256 0x0C
-#define VGIDS_SE_ALGOID_DST_ECDSA_P384 0x0D
-#define VGIDS_SE_ALGOID_DST_ECDSA_P512 0x0E
+// #define VGIDS_SE_ALGOID_DST_RSA_1024 0x06
+// #define VGIDS_SE_ALGOID_DST_RSA_2048 0x07
+// #define VGIDS_SE_ALGOID_DST_RSA_3072 0x08
+// #define VGIDS_SE_ALGOID_DST_RSA_4096 0x09
+// #define VGIDS_SE_ALGOID_DST_ECDSA_P192 0x0A
+// #define VGIDS_SE_ALGOID_DST_ECDSA_P224 0x0B
+// #define VGIDS_SE_ALGOID_DST_ECDSA_P256 0x0C
+// #define VGIDS_SE_ALGOID_DST_ECDSA_P384 0x0D
+// #define VGIDS_SE_ALGOID_DST_ECDSA_P512 0x0E
 
 #define VGIDS_DEFAULT_KEY_REF 0x81
 
@@ -294,8 +294,10 @@ create_failed:
 	return NULL;
 }
 
-static BOOL vgids_write_tlv(wStream* s, UINT16 tag, const void* data, DWORD dataSize)
+static BOOL vgids_write_tlv(wStream* s, UINT16 tag, const void* data, size_t dataSize)
 {
+	WINPR_ASSERT(dataSize <= UINT16_MAX);
+
 	/* A maximum of 5 additional bytes is needed */
 	if (!Stream_EnsureRemainingCapacity(s, dataSize + 5))
 	{
@@ -401,7 +403,8 @@ static BOOL vgids_ef_read_do(vgidsEF* ef, UINT16 doID, BYTE** data, DWORD* dataS
 		else
 		{
 			/* Skip DO */
-			Stream_SafeSeek(ef->data, doSize);
+			if (!Stream_SafeSeek(ef->data, doSize))
+				return FALSE;
 		}
 	}
 
@@ -422,7 +425,7 @@ static BOOL vgids_prepare_fstable(const vgidsFilesysTableEntry* fstable, DWORD n
                                   BYTE** outData, DWORD* outDataSize)
 {
 	/* Filesystem table:
-	    BYTE unkonwn: 0x01
+	    BYTE unknown: 0x01
 	    Array of vgidsFilesysTableEntry
 	*/
 	BYTE* data = malloc(sizeof(vgidsFilesysTableEntry) * numEntries + 1);
@@ -501,7 +504,7 @@ handle_error:
 	return FALSE;
 }
 
-static int get_rsa_key_size(const rdpPrivateKey* privateKey)
+static size_t get_rsa_key_size(const rdpPrivateKey* privateKey)
 {
 	WINPR_ASSERT(privateKey);
 
@@ -533,7 +536,7 @@ static BYTE vgids_get_algid(vgidsContext* p_Ctx)
 static BOOL vgids_prepare_keymap(vgidsContext* context, BYTE** outData, DWORD* outDataSize)
 {
 	/* Key map record table:
-	    BYTE unkonwn (count?): 0x01
+	    BYTE unknown (count?): 0x01
 	    Array of vgidsKeymapRecord
 	*/
 	BYTE* data = NULL;
@@ -876,7 +879,7 @@ static BOOL vgids_ins_getdata(vgidsContext* context, wStream* s, BYTE** response
 	vgids_reset_context_response(context);
 
 	/* build up file identifier */
-	fileId = ((UINT16)p1 << 8) | p2;
+	fileId = (UINT16)(((UINT16)p1 << 8) | p2);
 
 	/* Do we have a DO reference? */
 	switch (lc)
@@ -1029,7 +1032,7 @@ static BOOL vgids_ins_manage_security_environment(vgidsContext* context, wStream
 	Stream_Read_UINT8(s, context->currentSE.keyRef);
 
 create_response:
-	/* If an error occured reset SE */
+	/* If an error occurred reset SE */
 	if (status != ISO_STATUS_SUCCESS)
 		memset(&context->currentSE, 0, sizeof(context->currentSE));
 	return vgids_create_response(status, resultData, resultDataSize, response, responseSize);
@@ -1256,8 +1259,8 @@ static BOOL vgids_ins_perform_security_operation(vgidsContext* context, wStream*
 		if (!context->commandData)
 			return FALSE;
 	}
-	else
-		Stream_EnsureRemainingCapacity(context->commandData, lc);
+	else if (!Stream_EnsureRemainingCapacity(context->commandData, lc))
+		return FALSE;
 
 	Stream_Write(context->commandData, Stream_Pointer(s), lc);
 	Stream_SealLength(context->commandData);
@@ -1500,11 +1503,13 @@ BOOL vgids_init(vgidsContext* ctx, const char* cert, const char* privateKey, con
 		goto init_failed;
 
 	/* create masterfile */
+	// NOLINTNEXTLINE(clang-analyzer-unix.Malloc)
 	masterEF = vgids_ef_new(ctx, VGIDS_EFID_MASTER);
 	if (!masterEF)
 		goto init_failed;
 
 	/* create cardid file with cardid DO */
+	// NOLINTNEXTLINE(clang-analyzer-unix.Malloc)
 	cardidEF = vgids_ef_new(ctx, VGIDS_EFID_CARDID);
 	if (!cardidEF)
 		goto init_failed;
@@ -1513,6 +1518,7 @@ BOOL vgids_init(vgidsContext* ctx, const char* cert, const char* privateKey, con
 		goto init_failed;
 
 	/* create user common file */
+	// NOLINTNEXTLINE(clang-analyzer-unix.Malloc)
 	commonEF = vgids_ef_new(ctx, VGIDS_EFID_COMMON);
 	if (!commonEF)
 		goto init_failed;
@@ -1522,8 +1528,8 @@ BOOL vgids_init(vgidsContext* ctx, const char* cert, const char* privateKey, con
 		goto init_failed;
 
 	/* write container map DO */
-	const int size = get_rsa_key_size(ctx->privateKey);
-	if (size <= 0)
+	const size_t size = get_rsa_key_size(ctx->privateKey);
+	if ((size == 0) || (size > UINT16_MAX / 8))
 		goto init_failed;
 
 	cmrec.wKeyExchangeKeySizeBits = (WORD)size * 8;
