@@ -574,11 +574,13 @@ mbedtls_cipher_type_t winpr_mbedtls_get_cipher_type(int cipher)
 }
 #endif
 
+#if defined(WITH_FREERDP_3x_DEPRECATED)
 WINPR_CIPHER_CTX* winpr_Cipher_New(WINPR_CIPHER_TYPE cipher, WINPR_CRYPTO_OPERATION op,
                                    const void* key, const void* iv)
 {
 	return winpr_Cipher_NewEx(cipher, op, key, 0, iv, 0);
 }
+#endif
 
 WINPR_API WINPR_CIPHER_CTX* winpr_Cipher_NewEx(WINPR_CIPHER_TYPE cipher, WINPR_CRYPTO_OPERATION op,
                                                const void* key, WINPR_ATTR_UNUSED size_t keylen,
@@ -607,27 +609,6 @@ WINPR_API WINPR_CIPHER_CTX* winpr_Cipher_NewEx(WINPR_CIPHER_TYPE cipher, WINPR_C
 	if (!ctx->ectx)
 		goto fail;
 
-#if 0
-	if (keylen != 0)
-	{
-		WINPR_ASSERT(keylen <= INT32_MAX);
-		const int len = EVP_CIPHER_CTX_key_length(ctx->ectx);
-		if ((len > 0) && (len != keylen))
-		{
-			if (EVP_CIPHER_CTX_set_key_length(ctx->ectx, (int)keylen) != 1)
-				goto fail;
-		}
-	}
-
-	if (ivlen != 0)
-	{
-		WINPR_ASSERT(ivlen <= INT32_MAX);
-		const int len = EVP_CIPHER_CTX_iv_length(ctx->ectx);
-		if ((len > 0) && (ivlen != len))
-			goto fail;
-	}
-#endif
-
 	const int operation = (op == WINPR_ENCRYPT) ? 1 : 0;
 
 	if (EVP_CipherInit_ex(ctx->ectx, evp, NULL, key, iv, operation) != 1)
