@@ -18,7 +18,6 @@
  */
 
 #include <stdlib.h>
-#include <pkcs11-helper-1.0/pkcs11.h>
 
 #include <winpr/library.h>
 #include <winpr/assert.h>
@@ -29,6 +28,9 @@
 #include "../log.h"
 #include "ncrypt.h"
 
+/* https://github.com/latchset/pkcs11-headers/blob/main/public-domain/3.1/pkcs11.h */
+#include "pkcs11-headers/pkcs11.h"
+
 #define TAG WINPR_TAG("ncryptp11")
 
 #define MAX_SLOTS 64
@@ -42,6 +44,7 @@ typedef struct
 
 	HANDLE library;
 	CK_FUNCTION_LIST_PTR p11;
+	char* modulePath;
 } NCryptP11ProviderHandle;
 
 /** @brief a handle returned by NCryptOpenKey */
@@ -101,19 +104,26 @@ static CK_ATTRIBUTE public_key_filter[] = {
 	{ CKA_KEY_TYPE, &object_ktype_rsa, sizeof(object_ktype_rsa) }
 };
 
+static const char* CK_RV_error_string(CK_RV rv);
+
 static SECURITY_STATUS NCryptP11StorageProvider_dtor(NCRYPT_HANDLE handle)
 {
 	NCryptP11ProviderHandle* provider = (NCryptP11ProviderHandle*)handle;
-	CK_RV rv = 0;
+	CK_RV rv = CKR_OK;
 
-	WINPR_ASSERT(provider);
-	rv = provider->p11->C_Finalize(NULL);
-	if (rv != CKR_OK)
+	if (provider)
 	{
-	}
+		if (provider->p11 && provider->p11->C_Finalize)
+			rv = provider->p11->C_Finalize(NULL);
+		if (rv != CKR_OK)
+			WLog_WARN(TAG, "C_Finalize failed with %s [0x%08" PRIx32 "]", CK_RV_error_string(rv),
+			          rv);
 
-	if (provider->library)
-		FreeLibrary(provider->library);
+		free(provider->modulePath);
+
+		if (provider->library)
+			FreeLibrary(provider->library);
+	}
 
 	return winpr_NCryptDefault_dtor(handle);
 }
@@ -141,6 +151,7 @@ static BOOL attributes_have_unallocated_buffers(CK_ATTRIBUTE_PTR attributes, CK_
 
 static BOOL attribute_allocate_attribute_array(CK_ATTRIBUTE_PTR attribute)
 {
+	WINPR_ASSERT(attribute);
 	attribute->pValue = calloc(attribute->ulValueLen, sizeof(void*));
 	return !!attribute->pValue;
 }
@@ -190,13 +201,11 @@ static CK_RV object_load_attributes(NCryptP11ProviderHandle* provider, CK_SESSIO
                                     CK_OBJECT_HANDLE object, CK_ATTRIBUTE_PTR attributes,
                                     CK_ULONG count)
 {
-	CK_RV rv = 0;
-
 	WINPR_ASSERT(provider);
 	WINPR_ASSERT(provider->p11);
 	WINPR_ASSERT(provider->p11->C_GetAttributeValue);
 
-	rv = provider->p11->C_GetAttributeValue(session, object, attributes, count);
+	CK_RV rv = provider->p11->C_GetAttributeValue(session, object, attributes, count);
 
 	switch (rv)
 	{
@@ -213,8 +222,13 @@ static CK_RV object_load_attributes(NCryptP11ProviderHandle* provider, CK_SESSIO
 				return CKR_HOST_MEMORY;
 
 			rv = provider->p11->C_GetAttributeValue(session, object, attributes, count);
+			if (rv != CKR_OK)
+				WLog_WARN(TAG, "C_GetAttributeValue failed with %s [0x%08" PRIx32 "]",
+				          CK_RV_error_string(rv), rv);
 			break;
 		default:
+			WLog_WARN(TAG, "C_GetAttributeValue failed with %s [0x%08" PRIx32 "]",
+			          CK_RV_error_string(rv), rv);
 			return rv;
 	}
 
@@ -223,7 +237,12 @@ static CK_RV object_load_attributes(NCryptP11ProviderHandle* provider, CK_SESSIO
 		case CKR_ATTRIBUTE_SENSITIVE:
 		case CKR_ATTRIBUTE_TYPE_INVALID:
 		case CKR_BUFFER_TOO_SMALL:
-			WLog_ERR(TAG, "C_GetAttributeValue return %d even after buffer allocation", rv);
+			WLog_ERR(TAG,
+			         "C_GetAttributeValue failed with %s [0x%08" PRIx32
+			         "] even after buffer allocation",
+			         CK_RV_error_string(rv), rv);
+			break;
+		default:
 			break;
 	}
 	return rv;
@@ -324,16 +343,32 @@ static const char* CK_RV_error_string(CK_RV rv)
 		ERR_ENTRY(CKR_MUTEX_NOT_LOCKED);
 		ERR_ENTRY(CKR_FUNCTION_REJECTED);
 		default:
-			snprintf(generic_buffer, sizeof(generic_buffer), "unknown 0x%lx", rv);
+			(void)snprintf(generic_buffer, sizeof(generic_buffer), "unknown 0x%lx", rv);
 			return generic_buffer;
 	}
 #undef ERR_ENTRY
 }
 
+#define loge(tag, msg, rv, index, slot) \
+	log_((tag), (msg), (rv), (index), (slot), __FILE__, __func__, __LINE__)
+static void log_(const char* tag, const char* msg, CK_RV rv, CK_ULONG index, CK_SLOT_ID slot,
+                 const char* file, const char* fkt, size_t line)
+{
+	const DWORD log_level = WLOG_ERROR;
+	static wLog* log_cached_ptr = NULL;
+	if (!log_cached_ptr)
+		log_cached_ptr = WLog_Get(tag);
+	if (!WLog_IsLevelActive(log_cached_ptr, log_level))
+		return;
+
+	WLog_PrintMessage(log_cached_ptr, WLOG_MESSAGE_TEXT, log_level, line, file, fkt,
+	                  "%s for slot #%" PRIu32 "(%" PRIu32 "), rv=%s", msg, index, slot,
+	                  CK_RV_error_string(rv));
+}
+
 static SECURITY_STATUS collect_keys(NCryptP11ProviderHandle* provider, P11EnumKeysState* state)
 {
 	CK_OBJECT_HANDLE slotObjects[MAX_KEYS_PER_SLOT] = { 0 };
-	const char* step = NULL;
 
 	WINPR_ASSERT(provider);
 
@@ -352,28 +387,27 @@ static SECURITY_STATUS collect_keys(NCryptP11ProviderHandle* provider, P11EnumKe
 		CK_RV rv = p11->C_GetSlotInfo(state->slots[i], &slotInfo);
 		if (rv != CKR_OK)
 		{
-			WLog_ERR(TAG, "unable to retrieve information for slot #%d(%d)", i, state->slots[i]);
+			loge(TAG, "unable to retrieve information", rv, i, state->slots[i]);
 			continue;
 		}
 
 		fix_padded_string((char*)slotInfo.slotDescription, sizeof(slotInfo.slotDescription));
-		WLog_DBG(TAG, "collecting keys for slot #%d(%lu) descr='%s' flags=0x%x", i, state->slots[i],
-		         slotInfo.slotDescription, slotInfo.flags);
+		WLog_DBG(TAG, "collecting keys for slot #%" PRIu32 "(%" PRIu32 ") descr='%s' flags=0x%x", i,
+		         state->slots[i], slotInfo.slotDescription, slotInfo.flags);
 
 		/* this is a safety guard as we're supposed to have listed only readers with tokens in them
 		 */
 		if (!(slotInfo.flags & CKF_TOKEN_PRESENT))
 		{
-			WLog_INFO(TAG, "token not present for slot #%d(%d)", i, state->slots[i]);
+			WLog_INFO(TAG, "token not present for slot #%" PRIu32 "(%" PRIu32 ")", i,
+			          state->slots[i]);
 			continue;
 		}
 
 		WINPR_ASSERT(p11->C_GetTokenInfo);
 		rv = p11->C_GetTokenInfo(state->slots[i], &tokenInfo);
 		if (rv != CKR_OK)
-		{
-			WLog_INFO(TAG, "unable to retrieve token info for slot #%d(%d)", i, state->slots[i]);
-		}
+			loge(TAG, "unable to retrieve token info", rv, i, state->slots[i]);
 		else
 		{
 			fix_padded_string((char*)tokenInfo.label, sizeof(tokenInfo.label));
@@ -384,8 +418,9 @@ static SECURITY_STATUS collect_keys(NCryptP11ProviderHandle* provider, P11EnumKe
 		rv = p11->C_OpenSession(state->slots[i], CKF_SERIAL_SESSION, NULL, NULL, &session);
 		if (rv != CKR_OK)
 		{
-			WLog_ERR(TAG, "unable to openSession for slot #%d(%d), session=%p rv=%s", i,
-			         state->slots[i], session, CK_RV_error_string(rv));
+			WLog_ERR(TAG,
+			         "unable to openSession for slot #%" PRIu32 "(%" PRIu32 "), session=%p rv=%s",
+			         i, state->slots[i], session, CK_RV_error_string(rv));
 			continue;
 		}
 
@@ -394,9 +429,7 @@ static SECURITY_STATUS collect_keys(NCryptP11ProviderHandle* provider, P11EnumKe
 		if (rv != CKR_OK)
 		{
 			// TODO: shall it be fatal ?
-			WLog_ERR(TAG, "unable to initiate search for slot #%d(%d), rv=%s", i, state->slots[i],
-			         CK_RV_error_string(rv));
-			step = "C_FindObjectsInit";
+			loge(TAG, "unable to initiate search", rv, i, state->slots[i]);
 			goto cleanup_FindObjectsInit;
 		}
 
@@ -405,9 +438,7 @@ static SECURITY_STATUS collect_keys(NCryptP11ProviderHandle* provider, P11EnumKe
 		rv = p11->C_FindObjects(session, &slotObjects[0], ARRAYSIZE(slotObjects), &nslotObjects);
 		if (rv != CKR_OK)
 		{
-			WLog_ERR(TAG, "unable to findObjects for slot #%d(%d), rv=%s", i, state->slots[i],
-			         CK_RV_error_string(rv));
-			step = "C_FindObjects";
+			loge(TAG, "unable to findObjects", rv, i, state->slots[i]);
 			goto cleanup_FindObjects;
 		}
 
@@ -441,18 +472,12 @@ static SECURITY_STATUS collect_keys(NCryptP11ProviderHandle* provider, P11EnumKe
 		WINPR_ASSERT(p11->C_FindObjectsFinal);
 		rv = p11->C_FindObjectsFinal(session);
 		if (rv != CKR_OK)
-		{
-			WLog_ERR(TAG, "error during C_FindObjectsFinal for slot #%d(%d) (errorStep=%s), rv=%s",
-			         i, state->slots[i], step, CK_RV_error_string(rv));
-		}
+			loge(TAG, "error during C_FindObjectsFinal", rv, i, state->slots[i]);
 	cleanup_FindObjectsInit:
 		WINPR_ASSERT(p11->C_CloseSession);
 		rv = p11->C_CloseSession(session);
 		if (rv != CKR_OK)
-		{
-			WLog_ERR(TAG, "error closing session for slot #%d(%d) (errorStep=%s), rv=%s", i,
-			         state->slots[i], step, CK_RV_error_string(rv));
-		}
+			loge(TAG, "error closing session", rv, i, state->slots[i]);
 	}
 
 	return ERROR_SUCCESS;
@@ -460,19 +485,20 @@ static SECURITY_STATUS collect_keys(NCryptP11ProviderHandle* provider, P11EnumKe
 
 static BOOL convertKeyType(CK_KEY_TYPE k, LPWSTR dest, DWORD len, DWORD* outlen)
 {
-	DWORD retLen = 0;
 	const WCHAR* r = NULL;
+	size_t retLen = 0;
 
-#define ALGO_CASE(V, S) \
-	case V:             \
-		r = S;          \
+#define ALGO_CASE(V, S)                         \
+	case V:                                     \
+		r = S;                                  \
+		retLen = _wcsnlen((S), ARRAYSIZE((S))); \
 		break
 	switch (k)
 	{
 		ALGO_CASE(CKK_RSA, BCRYPT_RSA_ALGORITHM);
 		ALGO_CASE(CKK_DSA, BCRYPT_DSA_ALGORITHM);
 		ALGO_CASE(CKK_DH, BCRYPT_DH_ALGORITHM);
-		ALGO_CASE(CKK_ECDSA, BCRYPT_ECDSA_ALGORITHM);
+		ALGO_CASE(CKK_EC, BCRYPT_ECDSA_ALGORITHM);
 		ALGO_CASE(CKK_RC2, BCRYPT_RC2_ALGORITHM);
 		ALGO_CASE(CKK_RC4, BCRYPT_RC4_ALGORITHM);
 		ALGO_CASE(CKK_DES, BCRYPT_DES_ALGORITHM);
@@ -498,9 +524,11 @@ static BOOL convertKeyType(CK_KEY_TYPE k, LPWSTR dest, DWORD len, DWORD* outlen)
 	}
 #undef ALGO_CASE
 
-	retLen = _wcslen(r);
+	if (retLen > UINT32_MAX)
+		return FALSE;
+
 	if (outlen)
-		*outlen = retLen;
+		*outlen = (UINT32)retLen;
 
 	if (!r)
 	{
@@ -510,7 +538,7 @@ static BOOL convertKeyType(CK_KEY_TYPE k, LPWSTR dest, DWORD len, DWORD* outlen)
 	}
 	else
 	{
-		if (retLen + 1 < len)
+		if (retLen + 1 > len)
 		{
 			WLog_ERR(TAG, "target buffer is too small for algo name");
 			return FALSE;
@@ -518,7 +546,7 @@ static BOOL convertKeyType(CK_KEY_TYPE k, LPWSTR dest, DWORD len, DWORD* outlen)
 
 		if (dest)
 		{
-			memcpy(dest, r, retLen * 2);
+			memcpy(dest, r, sizeof(WCHAR) * retLen);
 			dest[retLen] = 0;
 		}
 	}
@@ -537,25 +565,25 @@ static void wprintKeyName(LPWSTR str, CK_SLOT_ID slotId, CK_BYTE* id, CK_ULONG i
 
 	bytePtr = ((CK_BYTE*)&slotId);
 	for (CK_ULONG i = 0; i < sizeof(slotId); i++, bytePtr++, ptr += 2)
-		snprintf(ptr, 3, "%.2x", *bytePtr);
+		(void)snprintf(ptr, 3, "%.2x", *bytePtr);
 
 	*ptr = '\\';
 	ptr++;
 
 	for (CK_ULONG i = 0; i < idLen; i++, id++, ptr += 2)
-		snprintf(ptr, 3, "%.2x", *id);
+		(void)snprintf(ptr, 3, "%.2x", *id);
 
-	ConvertUtf8NToWChar(asciiName, ARRAYSIZE(asciiName), str,
-	                    strnlen(asciiName, ARRAYSIZE(asciiName)) + 1);
+	(void)ConvertUtf8NToWChar(asciiName, ARRAYSIZE(asciiName), str,
+	                          strnlen(asciiName, ARRAYSIZE(asciiName)) + 1);
 }
 
 static size_t parseHex(const char* str, const char* end, CK_BYTE* target)
 {
-	int ret = 0;
+	size_t ret = 0;
 
 	for (; str != end && *str; str++, ret++, target++)
 	{
-		CK_BYTE v = 0;
+		int v = 0;
 		if (*str <= '9' && *str >= '0')
 		{
 			v = (*str - '0');
@@ -595,7 +623,7 @@ static size_t parseHex(const char* str, const char* end, CK_BYTE* target)
 			return 0;
 		}
 
-		*target = v;
+		*target = v & 0xFF;
 	}
 	return ret;
 }
@@ -632,7 +660,7 @@ static SECURITY_STATUS parseKeyName(LPCWSTR pszKeyName, CK_SLOT_ID* slotId, CK_B
 
 static SECURITY_STATUS NCryptP11EnumKeys(NCRYPT_PROV_HANDLE hProvider, LPCWSTR pszScope,
                                          NCryptKeyName** ppKeyName, PVOID* ppEnumState,
-                                         DWORD dwFlags)
+                                         WINPR_ATTR_UNUSED DWORD dwFlags)
 {
 	NCryptP11ProviderHandle* provider = (NCryptP11ProviderHandle*)hProvider;
 	P11EnumKeysState* state = (P11EnumKeysState*)*ppEnumState;
@@ -692,8 +720,10 @@ static SECURITY_STATUS NCryptP11EnumKeys(NCRYPT_PROV_HANDLE hProvider, LPCWSTR p
 		rv = provider->p11->C_GetSlotList(CK_TRUE, NULL, &state->nslots);
 		if (rv != CKR_OK)
 		{
+			free(state);
 			/* TODO: perhaps convert rv to NTE_*** errors */
-			WLog_WARN(TAG, "C_GetSlotList failed with %u", rv);
+			WLog_WARN(TAG, "C_GetSlotList failed with %s [0x%08" PRIx32 "]", CK_RV_error_string(rv),
+			          rv);
 			return NTE_FAIL;
 		}
 
@@ -705,7 +735,8 @@ static SECURITY_STATUS NCryptP11EnumKeys(NCRYPT_PROV_HANDLE hProvider, LPCWSTR p
 		{
 			free(state);
 			/* TODO: perhaps convert rv to NTE_*** errors */
-			WLog_WARN(TAG, "C_GetSlotList failed with %u", rv);
+			WLog_WARN(TAG, "C_GetSlotList failed with %s [0x%08" PRIx32 "]", CK_RV_error_string(rv),
+			          rv);
 			return NTE_FAIL;
 		}
 
@@ -743,6 +774,9 @@ static SECURITY_STATUS NCryptP11EnumKeys(NCRYPT_PROV_HANDLE hProvider, LPCWSTR p
 			{
 				WINPR_ASSERT(provider->p11->C_CloseSession);
 				rv = provider->p11->C_CloseSession(currentSession);
+				if (rv != CKR_OK)
+					WLog_WARN(TAG, "C_CloseSession failed with %s [0x%08" PRIx32 "]",
+					          CK_RV_error_string(rv), rv);
 				currentSession = (CK_SESSION_HANDLE)NULL;
 			}
 
@@ -751,7 +785,8 @@ static SECURITY_STATUS NCryptP11EnumKeys(NCRYPT_PROV_HANDLE hProvider, LPCWSTR p
 			                                  &currentSession);
 			if (rv != CKR_OK)
 			{
-				WLog_ERR(TAG, "unable to openSession for slot %d", key->slotId);
+				WLog_ERR(TAG, "C_OpenSession failed with %s [0x%08" PRIx32 "] for slot %d",
+				         CK_RV_error_string(rv), rv, key->slotId);
 				continue;
 			}
 			currentSlot = key->slotId;
@@ -763,7 +798,8 @@ static SECURITY_STATUS NCryptP11EnumKeys(NCRYPT_PROV_HANDLE hProvider, LPCWSTR p
 		                                      ARRAYSIZE(certificateFilter));
 		if (rv != CKR_OK)
 		{
-			WLog_ERR(TAG, "unable to initiate search for slot %d", key->slotId);
+			WLog_ERR(TAG, "C_FindObjectsInit failed with %s [0x%08" PRIx32 "] for slot %d",
+			         CK_RV_error_string(rv), rv, key->slotId);
 			continue;
 		}
 
@@ -771,7 +807,8 @@ static SECURITY_STATUS NCryptP11EnumKeys(NCRYPT_PROV_HANDLE hProvider, LPCWSTR p
 		rv = provider->p11->C_FindObjects(currentSession, &certObject, 1, &ncertObjects);
 		if (rv != CKR_OK)
 		{
-			WLog_ERR(TAG, "unable to findObjects for slot %d", currentSlot);
+			WLog_ERR(TAG, "C_FindObjects failed with %s [0x%08" PRIx32 "] for slot %d",
+			         CK_RV_error_string(rv), rv, currentSlot);
 			goto cleanup_FindObjects;
 		}
 
@@ -783,7 +820,7 @@ static SECURITY_STATUS NCryptP11EnumKeys(NCRYPT_PROV_HANDLE hProvider, LPCWSTR p
 			    (1 + (sizeof(key->slotId) * 2) /*slotId*/ + 1 + (key->idLen * 2) + 1) * 2;
 
 			convertKeyType(key->keyType, NULL, 0, &algoSz);
-			KEYNAME_SZ += (algoSz + 1) * 2;
+			KEYNAME_SZ += (1ULL + algoSz) * 2ULL;
 
 			keyName = calloc(1, sizeof(*keyName) + KEYNAME_SZ);
 			if (!keyName)
@@ -803,6 +840,9 @@ static SECURITY_STATUS NCryptP11EnumKeys(NCRYPT_PROV_HANDLE hProvider, LPCWSTR p
 	cleanup_FindObjects:
 		WINPR_ASSERT(provider->p11->C_FindObjectsFinal);
 		rv = provider->p11->C_FindObjectsFinal(currentSession);
+		if (rv != CKR_OK)
+			WLog_ERR(TAG, "C_FindObjectsFinal failed with %s [0x%08" PRIx32 "]",
+			         CK_RV_error_string(rv), rv);
 
 		if (keyName)
 		{
@@ -891,10 +931,10 @@ static SECURITY_STATUS get_piv_container_name(NCryptP11KeyHandle* key, const BYT
 	p = Stream_Buffer(&s);
 
 	/* Construct the value Windows would use for a PIV key's container name */
-	snprintf(container_name, PIV_CONTAINER_NAME_LEN + 1,
-	         "%.2x%.2x%.2x%.2x-%.2x%.2x-%.2x%.2x-%.2x%.2x-%.2x%.2x%.2x%.2x%.2x%.2x", p[3], p[2],
-	         p[1], p[0], p[5], p[4], p[7], p[6], p[8], p[9], p[10], p[11], p[12], piv_tag[0],
-	         piv_tag[1], piv_tag[2]);
+	(void)snprintf(container_name, PIV_CONTAINER_NAME_LEN + 1,
+	               "%.2x%.2x%.2x%.2x-%.2x%.2x-%.2x%.2x-%.2x%.2x-%.2x%.2x%.2x%.2x%.2x%.2x", p[3],
+	               p[2], p[1], p[0], p[5], p[4], p[7], p[6], p[8], p[9], p[10], p[11], p[12],
+	               piv_tag[0], piv_tag[1], piv_tag[2]);
 
 	/* And convert it to UTF-16 */
 	union
@@ -939,7 +979,8 @@ static SECURITY_STATUS check_for_piv_container_name(NCryptP11KeyHandle* key, BYT
 
 static SECURITY_STATUS NCryptP11KeyGetProperties(NCryptP11KeyHandle* keyHandle,
                                                  NCryptKeyGetPropertyEnum property, PBYTE pbOutput,
-                                                 DWORD cbOutput, DWORD* pcbResult, DWORD dwFlags)
+                                                 DWORD cbOutput, DWORD* pcbResult,
+                                                 WINPR_ATTR_UNUSED DWORD dwFlags)
 {
 	SECURITY_STATUS ret = NTE_FAIL;
 	CK_RV rv = 0;
@@ -977,7 +1018,10 @@ static SECURITY_STATUS NCryptP11KeyGetProperties(NCryptP11KeyHandle* keyHandle,
 
 #define SLOT_DESC_SZ sizeof(slotInfo.slotDescription)
 			fix_padded_string((char*)slotInfo.slotDescription, SLOT_DESC_SZ);
-			*pcbResult = 2 * (strnlen((char*)slotInfo.slotDescription, SLOT_DESC_SZ) + 1);
+			const size_t len = 2ULL * (strnlen((char*)slotInfo.slotDescription, SLOT_DESC_SZ) + 1);
+			if (len > UINT32_MAX)
+				return NTE_BAD_DATA;
+			*pcbResult = (UINT32)len;
 			if (pbOutput)
 			{
 				union
@@ -1004,8 +1048,12 @@ static SECURITY_STATUS NCryptP11KeyGetProperties(NCryptP11KeyHandle* keyHandle,
 
 				if (cbOutput < 4)
 					return NTE_NO_MEMORY;
-
-				*ptr = keyHandle->slotId;
+				if (keyHandle->slotId > UINT32_MAX)
+				{
+					ret = NTE_BAD_DATA;
+					goto out_final;
+				}
+				*ptr = (UINT32)keyHandle->slotId;
 			}
 			return ERROR_SUCCESS;
 		}
@@ -1056,7 +1104,12 @@ static SECURITY_STATUS NCryptP11KeyGetProperties(NCryptP11KeyHandle* keyHandle,
 				// TODO: do a kind of translation from CKR_* to NTE_*
 			}
 
-			*pcbResult = certValue.ulValueLen;
+			if (certValue.ulValueLen > UINT32_MAX)
+			{
+				ret = NTE_BAD_DATA;
+				goto out_final;
+			}
+			*pcbResult = (UINT32)certValue.ulValueLen;
 			ret = ERROR_SUCCESS;
 			break;
 		}
@@ -1150,7 +1203,8 @@ static SECURITY_STATUS NCryptP11GetProperty(NCRYPT_HANDLE hObject, NCryptKeyGetP
 }
 
 static SECURITY_STATUS NCryptP11OpenKey(NCRYPT_PROV_HANDLE hProvider, NCRYPT_KEY_HANDLE* phKey,
-                                        LPCWSTR pszKeyName, DWORD dwLegacyKeySpec, DWORD dwFlags)
+                                        LPCWSTR pszKeyName, WINPR_ATTR_UNUSED DWORD dwLegacyKeySpec,
+                                        WINPR_ATTR_UNUSED DWORD dwFlags)
 {
 	SECURITY_STATUS ret = 0;
 	CK_SLOT_ID slotId = 0;
@@ -1189,11 +1243,7 @@ static SECURITY_STATUS initialize_pkcs11(HANDLE handle,
 	ret = (NCryptP11ProviderHandle*)ncrypt_new_handle(
 	    WINPR_NCRYPT_PROVIDER, sizeof(*ret), NCryptP11GetProperty, NCryptP11StorageProvider_dtor);
 	if (!ret)
-	{
-		if (handle)
-			FreeLibrary(handle);
 		return NTE_NO_MEMORY;
-	}
 
 	ret->library = handle;
 	ret->baseProvider.enumKeysFn = NCryptP11EnumKeys;
@@ -1206,6 +1256,7 @@ static SECURITY_STATUS initialize_pkcs11(HANDLE handle,
 		goto fail;
 	}
 
+	WINPR_ASSERT(ret->p11);
 	WINPR_ASSERT(ret->p11->C_Initialize);
 	rv = ret->p11->C_Initialize(NULL);
 	if (rv != CKR_OK)
@@ -1223,56 +1274,35 @@ fail:
 }
 
 SECURITY_STATUS NCryptOpenP11StorageProviderEx(NCRYPT_PROV_HANDLE* phProvider,
-                                               LPCWSTR pszProviderName, DWORD dwFlags,
-                                               LPCSTR* modulePaths)
+                                               WINPR_ATTR_UNUSED LPCWSTR pszProviderName,
+                                               WINPR_ATTR_UNUSED DWORD dwFlags, LPCSTR* modulePaths)
 {
 	SECURITY_STATUS status = ERROR_INVALID_PARAMETER;
-#if defined(__LP64__) || (defined(__x86_64__) && !defined(__ILP32__)) || defined(_M_X64) || \
-    defined(__ia64) || defined(_M_IA64) || defined(__aarch64__) || defined(__powerpc64__)
-#define LIBS64
-#endif
-
-	LPCSTR openscPaths[] = { "opensc-pkcs11.so", /* In case winpr is installed in system paths */
-#ifdef __APPLE__
-		                     "/usr/local/lib/pkcs11/opensc-pkcs11.so",
-#else
-	/* linux and UNIXes */
-#ifdef LIBS64
-		                     "/usr/lib/x86_64-linux-gnu/pkcs11/opensc-pkcs11.so", /* Ubuntu/debian
-		                                                                           */
-		                     "/lib64/pkcs11/opensc-pkcs11.so",                    /* Fedora */
-#else
-		                     "/usr/lib/i386-linux-gnu/opensc-pkcs11.so", /* debian */
-		                     "/lib32/pkcs11/opensc-pkcs11.so",           /* Fedora */
-#endif
-#endif
-		                     NULL };
+	LPCSTR defaultPaths[] = { "p11-kit-proxy.so", "opensc-pkcs11.so", NULL };
 
 	if (!phProvider)
 		return ERROR_INVALID_PARAMETER;
 
-#if defined(WITH_OPENSC_PKCS11_LINKED)
 	if (!modulePaths)
-		return initialize_pkcs11(NULL, C_GetFunctionList, phProvider);
-#endif
-
-	if (!modulePaths)
-		modulePaths = openscPaths;
+		modulePaths = defaultPaths;
 
 	while (*modulePaths)
 	{
-		HANDLE library = LoadLibrary(*modulePaths);
+		const char* modulePath = *modulePaths++;
+		HANDLE library = LoadLibrary(modulePath);
 		typedef CK_RV (*c_get_function_list_t)(CK_FUNCTION_LIST_PTR_PTR);
-		c_get_function_list_t c_get_function_list = NULL;
+		NCryptP11ProviderHandle* provider = NULL;
 
-		WLog_DBG(TAG, "Trying pkcs11-helper module '%s'", *modulePaths);
+		WLog_DBG(TAG, "Trying pkcs11 module '%s'", modulePath);
 		if (!library)
 		{
 			status = NTE_PROV_DLL_NOT_FOUND;
 			goto out_load_library;
 		}
 
-		c_get_function_list = (c_get_function_list_t)GetProcAddress(library, "C_GetFunctionList");
+		c_get_function_list_t c_get_function_list =
+		    GetProcAddressAs(library, "C_GetFunctionList", c_get_function_list_t);
+
 		if (!c_get_function_list)
 		{
 			status = NTE_PROV_TYPE_ENTRY_BAD;
@@ -1286,12 +1316,30 @@ SECURITY_STATUS NCryptOpenP11StorageProviderEx(NCRYPT_PROV_HANDLE* phProvider,
 			goto out_load_library;
 		}
 
-		WLog_DBG(TAG, "module '%s' loaded", *modulePaths);
+		provider = (NCryptP11ProviderHandle*)*phProvider;
+		provider->modulePath = _strdup(modulePath);
+		if (!provider->modulePath)
+		{
+			status = NTE_NO_MEMORY;
+			goto out_load_library;
+		}
+
+		WLog_DBG(TAG, "module '%s' loaded", modulePath);
 		return ERROR_SUCCESS;
 
 	out_load_library:
-		modulePaths++;
+		if (library)
+			FreeLibrary(library);
 	}
 
 	return status;
 }
+
+const char* NCryptGetModulePath(NCRYPT_PROV_HANDLE phProvider)
+{
+	NCryptP11ProviderHandle* provider = (NCryptP11ProviderHandle*)phProvider;
+
+	WINPR_ASSERT(provider);
+
+	return provider->modulePath;
+}
